!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
                          SUBROUTINE EPS
C     ******************************************************************
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .
C SUBPROGRAM:    EPS         
C   PRGRMMR: JANJIC          ORG: W/NMC2     DATE: 9?-??-??
C
C ABSTRACT:
C     EPS COMPUTES THE VERTICAL AND HORIZONTAL ADVECTION OF DZ/DT
C
C PROGRAM HISTORY LOG:
C   9?-??-??  JANJIC     - ORIGINATOR
C   00-01-05  BLACK      - DISTRIBUTED MEMORY AND THREADS
C
C USAGE: CALL EPS FROM MAIN PROGRAM
C   INPUT ARGUMENT LIST:
C       NONE
C
C   OUTPUT ARGUMENT LIST:
C     NONE
C
C   OUTPUT FILES:
C     NONE
C
C   SUBPROGRAMS CALLED:
C
C     UNIQUE: NONE
C
C     LIBRARY: NONE
C
C   COMMON BLOCKS: CTLBLK
C                  LOOPS
C                  MASKS
C                  DYNAM
C                  CONTIN
C                  VRBLS
C                  PVRBLS
C                  NHYDRO
C                  INDX
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 90
C   MACHINE : IBM SP
C$$$
C***********************************************************************
      INCLUDE "parmeta"
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C-----------------------------------------------------------------------
                             P A R A M E T E R
     &(IM1=IM-1,JAM=6+2*(JM-10)
     &,JAMD=(JAM*2-10)*3,LP1=LM+1)
C-----------------------------------------------------------------------
                             P A R A M E T E R
     &(NTSHY=3
     &,G=9.806,CP=1004.6,CAPA=287.04/CP,GMA=-287.04*(1.-CAPA)/2.
     &,cappa=287.04/1004.6)
C-----------------------------------------------------------------------
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA,TOP,BOT
C-----------------------------------------------------------------------
      INCLUDE "CTLBLK.comm"
C-----------------------------------------------------------------------
      INCLUDE "LOOPS.comm"
C-----------------------------------------------------------------------
      INCLUDE "MASKS.comm"
C-----------------------------------------------------------------------
      INCLUDE "DYNAM.comm"
C-----------------------------------------------------------------------
      INCLUDE "CONTIN.comm"
C-----------------------------------------------------------------------
      INCLUDE "VRBLS.comm"
C-----------------------------------------------------------------------
      INCLUDE "PVRBLS.comm"
C-----------------------------------------------------------------------
      INCLUDE "NHYDRO.comm"
!-----------------------------------------------------------------------
      include "CLDWTR.comm"
C-----------------------------------------------------------------------
      INCLUDE "INDX.comm"
C-------------LOCAL VARIABLES REQUIRING DECLARATION---------------------
                             R E A L
     & rdpp(lm),c0(lm),b1(lm),b2(lm),b3(lm)
     &,PONE(LM+1),PSTR(LM+1),PNP1(LM+1),COFF(LM+1)
     &,CHI(LM+1)
C-----------------------------------------------------------------------
                             R E A L
     & TTB (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ANE (IDIM1:IDIM2,JDIM1:JDIM2)   ,ASE (IDIM1:IDIM2,JDIM1:JDIM2)
C-----------------------------------------------------------------------
                             R E A L
     & DPDE  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,UNED  (IDIM1:IDIM2,JDIM1:JDIM2),USED  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,wew   (idim1:idim2,jdim1:jdim2),wns   (idim1:idim2,jdim1:jdim2)
     &,wne   (idim1:idim2,jdim1:jdim2),wse   (idim1:idim2,jdim1:jdim2)
C-----------------------------------------------------------------------
                             R E A L
     & GSUMS(2,LM),XSUMS(2,LM)
C-----------------------------------------------------------------------
C***
C***  THE FOLLOWING ARE USED FOR TIMIMG PURPOSES ONLY
C***
      real*8 timef
      real nhb_tim,mpp_tim,init_tim
      common/timing/surfce_tim,nhb_tim,res_tim,exch_tim
C***********************************************************************
C-----------------------------------------------------------------------
      IF(NTSD.LE.NTSHY.OR.HYDRO)THEN
C***
!$omp parallel do
        DO J=MYJS_P2,MYJE_P2
        DO I=MYIS_P1,MYIE_P1
          PINT(I,J,1)=PT
        ENDDO
        ENDDO
C
        DO L=1,LM
!$omp parallel do
          DO J=MYJS_P2,MYJE_P2
          DO I=MYIS_P1,MYIE_P1
            DWDT(I,J,L)=1.
            PDWDT(I,J,L)=1.
            pint(i,j,l+1)=deta1(l)*pdtop+deta2(l)*pdsl(i,j)+pint(i,j,l)
          ENDDO
          ENDDO
        ENDDO
C***
        RETURN
C***
      ENDIF
C-----------------------------------------------------------------------
      ADDT=DT
      RDT=1./ADDT
C-----------------TIME TENDENCY-----------------------------------------
!$omp parallel do
      DO L=1,LM
        DO J=MYJS_P1,MYJE_P1
        DO I=MYIS_P1,MYIE_P1
          DWDT(I,J,L)=(W(I,J,L)-DWDT(I,J,L))*HTM(I,J,L)*HBM2(I,J)*RDT
        ENDDO
        ENDDO
      ENDDO
C
C-----------------------------------------------------------------------
C-----------------VERTICAL ADVECTION------------------------------------
C-----------------------------------------------------------------------
!$omp parallel do
      do j=myjs,myje
      do i=myis,myie
        ttb(i,j)=0.
      enddo
      enddo
C
      do l=1,lm-1
!$omp parallel do
        do j=myjs2,myje2
        do i=myis1,myie1
          ttal=(w(i,j,l+1)-w(i,j,l))*petdt(i,j,l)*0.5
          dwdt(i,j,l)=(ttal+ttb(i,j))
     &               /(deta1(l)*pdtop+deta2(l)*pdslo(i,j))
     &               +dwdt(i,j,l)
          ttb(i,j)=ttal
        enddo
        enddo
      enddo
C
!$omp parallel do
      do j=myjs2,myje2
      do i=myis1,myie1
        dwdt(i,j,lm)=ttb(i,j)/(deta1(lm)*pdtop+deta2(lm)*pdslo(i,j))
     &              +dwdt(i,j,lm)
      enddo
      enddo
C-----------------------------------------------------------------------
C---------------------HORIZONTAL ADVECTION------------------------------
C-----------------------------------------------------------------------
!$omp parallel do
!$omp& private (dpde,ihh,ihl,ivh,ivl,ix,jx,udy,uned,used,
!$omp&          vdx,wew,wne,wns,wse)
      DO 200 L=1,LM
C-----------------------------------------------------------------------
      DO J=MYJS_P3,MYJE_P3
      DO I=MYIS_P3,MYIE_P3
        dpde(i,j)=deta1(l)*pdtop+deta2(l)*pdslo(i,j)
      ENDDO
      ENDDO
!--------------MASS FLUXES AND MASS POINTS ADVECTION COMPONENTS---------
              do j=myjs1_p3,myje1_p3
          do i=myis_p3,myie_p3
      wew(i,j)=few(i,j,l)
     2        *(w   (i+ive(j),j  ,l)-w   (i+ivw(j),j  ,l))
      wns(i,j)=fns(i,j,l)
     2        *(w   (i       ,j+1,l)-w   (i       ,j-1,l))
          enddo
              enddo
!--------------DIAGONAL FLUXES AND DIAGONALLY AVERAGED WIND-------------
              do j=myjs1_p2,myje2_p2
          do i=myis_p2,myie1_p2
      wne(i,j)=fne(i,j,l)
     2        *(w   (i+ihe(j),j+1,l)-w  (i,j,l))
          enddo
              enddo
              do j=myjs2_p2,myje1_p2
          do i=myis_p2,myie1_p2
      wse(i,j)=fse(i,j,l)
     2        *(w   (i+ihe(j),j-1,l)-w  (i,j,l))
          enddo
              enddo
!--------------ADVECTION OF W-------------------------------------------
              do j=myjs3,myje3
          do i=myis2,myie2
      dwdt(i,j,l)=
     1          -(wew(i+ihw(j),j  )+wew(i+ihe(j),j  )
     1           +wns(i       ,j-1)+wns(i       ,j+1)
     2           +wne(i+ihw(j),j-1)+wne(i       ,j  )
     2           +wse(i       ,j  )+wse(i+ihw(j),j+1))
     3           *fad(i,j)*htm(i,j,l)*hbm3(i,j)/(dpde(i,j)*dt)
     4          +dwdt(i,j,l)
          enddo
              enddo
C-----------------------------------------------------------------------
  200 CONTINUE
C------------------------------------------------------------------------
!zj      WA=0.075
!      wa=0.00
!          if(wa.gt.0.) then
!-----------------------------------------------------------------------
!$omp parallel do private (ane,ase)
!      DO 500 L=1,LM
C
!      DO J=MYJS1_P1,MYJE2_P1
!      DO I=MYIS_P1,MYIE1_P1
!        ANE(I,J)=(DWDT(I+IHE(J),J+1,L)-DWDT(I,J,L))
!     &           *HTM(I,J,L)*HTM(I+IHE(J),J+1,L)
!      ENDDO
!      ENDDO
C
!      DO J=MYJS2_P1,MYJE1_P1
!      DO I=MYIS_P1,MYIE1_P1
!        ASE(I,J)=(DWDT(I+IHE(J),J-1,L)-DWDT(I,J,L))
!     &           *HTM(I+IHE(J),J-1,L)*HTM(I,J,L)
!      ENDDO
!      ENDDO
C
!      DO J=MYJS2,MYJE2
!      DO I=MYIS1,MYIE1
!        DWDT(I,J,L)=(ANE(I,J)-ANE(I+IHW(J),J-1)
!     1              +ASE(I,J)-ASE(I+IHW(J),J+1))*WA*HBM2(I,J)
!     2              +DWDT(I,J,L)
!      ENDDO
!      ENDDO
C-----------------------------------------------------------------------
!  500 CONTINUE
!          endif
C-----------------------------------------------------------------------
!      WP=0.075
      wp=0.
      rg=1./g
C-----------------------------------------------------------------------
!$omp parallel do 
!$omp& private (dwdtmn,dwdtmx,dwdtt,hm,kn,kp)
      DO 525 L=1,LM
C-----------------------------------------------------------------------
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        dwdtt=dwdt(i,j,l)*htm(i,j,l)
        dwdtt=max(dwdtt,dwdtmn(i,j))
        dwdtt=min(dwdtt,dwdtmx(i,j))
!
        dwdt(i,j,l)=(dwdtt*rg+1.)*(1.-wp)+pdwdt(i,j,l)*wp
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
  525 CONTINUE
C-----------------------------------------------------------------------
      GDT=G*DT
      GDT2=GDT*GDT
      WGHT=0.35
      ffc=-r/gdt2
C-----------------------------------------------------------------------
!$omp parallel do
!$omp& private (c0,b1,b2,b3,chi,coff,dpstr,dp,tfc,fcc,rdpdn,
!$omp&          pnp1,pone,pp1,pstr,rdpdup,tmp,dptu,dptl,dwdtt,wil,delp,
!$omp&          hbm3ij)
C-----------------------------------------------------------------------
      DO 600 J=MYJS3,MYJE3
      DO 600 I=MYIS2,MYIE2
C-----------------------------------------------------------------------
      LMP=LMH(I,J)
      PDP=PDSL(I,J)
!-----------------------------------------------------------------------
      pone(1)=pt
      pstr(1)=pt
      pnp1(1)=pt
      chi(1)=0.
!
          do l=2,lmp+1
      chi(l)=0.
      dppl=deta1(l-1)*pdtop+deta2(l-1)*pdp
      rdpp(l-1)=1./dppl
      pone(l)=pint(i,j,l)
      dpstr=dwdt(i,j,l-1)*dppl
      pstr(l)=pstr(l-1)+dpstr
      pp1=pnp1(l-1)+dpstr
      pnp1(l)=(pp1-pone(l))*wght+pone(l)
      tfc=q(i,j,l-1)*0.608+(1.-cwm(i,j,l-1))
      ttfc=-cappa*tfc+1.
      coff(l-1)=t(i,j,l-1)*ttfc*tfc*dppl*ffc
     &         /((pnp1(l-1)+pnp1(l))*(pnp1(l-1)+pnp1(l)))
          enddo
!
      pstrup=-(pstr(1)+pstr(2)-pone(1)-pone(2))*coff(1)
!
          do l=2,lmp
      rdpdn=rdpp(l)
      rdpup=rdpp(l-1)
!
      pstrdn=-(pstr(l)+pstr(l+1)-pone(l)-pone(l+1))*coff(l)
!
      b1(l)=coff(l-1)+rdpup
      b2(l)=(coff(l-1)+coff(l))-(rdpup+rdpdn)
      b3(l)=coff(l)+rdpdn
!zj      c0(l)=-((pstr(l-1)+pstr(l)-pone(l-1)-pone(l))*coff(l-1)
!zj     &       +(pstr(l)+pstr(l+1)-pone(l)-pone(l+1))*coff(l))
      c0(l)=pstrup+pstrdn
!
      pstrup=pstrdn
          enddo
!
      b1(2)=0.
      b2(lmp)=b2(lmp)+b3(lmp)
!---------------elimination---------------------------------------------
          do l=3,lmp
      tmp=-b1(l)/b2(l-1)
      b2(l)=b3(l-1)*tmp+b2(l)
      c0(l)=c0(l-1)*tmp+c0(l)
          enddo
!
      chi(1)=0.
!---------------back substitution--------------------------------------
      chi(lmp)=c0(lmp)/b2(lmp)
      chi(lmp+1)=chi(lmp)
!
          do l=lmp-1,2,-1
      chi(l)=(-b3(l)*chi(l+1)+c0(l))/b2(l)
          enddo
!-----------------------------------------------------------------------
      hbm3ij=hbm3(i,j)
      dptu=0.
      fct=0.5/cp*hbm3ij
!
          do l=1,lmp
      dptl=(chi(l+1)+pstr(l+1)-pint(i,j,l+1))*hbm3ij
      pint(i,j,l+1)=pint(i,j,l+1)+dptl
      t(i,j,l)=(dptu+dptl)*rtop(i,j,l)*fct+t(i,j,l)
      delp=(pint(i,j,l+1)-pint(i,j,l))*rdpp(l)
      w(i,j,l)=((delp-dwdt(i,j,l))*gdt+w(i,j,l))*hbm3ij
      dwdt(i,j,l)=(delp-1.)*hbm3ij+1.
!
      dptu=dptl
         enddo
C-----------------------------------------------------------------------
  600 CONTINUE
C-----------------------------------------------------------------------
                             RETURN
                             END
