      SUBROUTINE FILT25(ZI,TM,IPASS)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK                                    
C                .      .    .                                       .  
C SUBPROGRAM:    FILTER      FILTERS THE ARRAY ZI                       
C   PRGMMR: DIMEGO           ORG: W/NP22     DATE: 86-07-18             
C                                                                       
C ABSTRACT: FILTERS AN ARRAY USING A 25PT BLECK FILTER IN THE           
C   INTERIOR OF THE DOMAIN                                              
C                                                                       
C PROGRAM HISTORY LOG:                                                  
C   86-07-18  G DIMEGO - ORIGINATOR
C   88-09-23  B SCHMIDT - ADDED THE DOCBLOCK                              
C   90-11-27  G DIMEGO - LEFT Z AS INTERNAL WORK ARRAY ON CRAY
C   93-06-21  R TREADON - STREAMLINED CODE
C   95-06-21  T BLACK - MODIFIED FOR THE E-GRID
C   99-08-25  T BLACK - MODIFIED FOR DISTRIBUTED MEMORY
C                                                                       
C USAGE:    CALL FILTER (IDIM1,IDIM2,JDIM1,JDIM2,ZI,IPASS)                        
C   INPUT ARGUMENT LIST:                                                
C     ZI       - ARRAY CONTAINING THE ARRAY TO BE FILTERED              
C     TM       - ARRAY CONTAINING THE TOPOGRAPHY MASK
C     IPASS    - NUMBER OF PASSES THROUGH THE FILTER                    
C                                                                       
C   OUTPUT ARGUMENT LIST:                                               
C     ZI       - ARRAY CONTAINING THE FILTERED FIELD                    
C                                                                       
C ATTRIBUTES:                                                           
C   LANGUAGE: FORTRAN 90
C    MACHINE: IBM SP                                                           
C                                                                       
C$$$                                                                    
C----------------------------------------------------------------
      INCLUDE "parmeta"
      INCLUDE "mpp.h"
C----------------------------------------------------------------
      INCLUDE "INDX.comm"
C----------------------------------------------------------------
      REAL ZI(IDIM1:IDIM2,JDIM1:JDIM2),Z(IDIM1:IDIM2,JDIM1:JDIM2)
      REAL TM(IDIM1:IDIM2,JDIM1:JDIM2)
C
      DATA CF1/0.279372/,CF2/0.171943/,CF3/-0.006918/
     1,    CF4/0.077458/,CF5/-0.024693/,CF6/-0.012940/
C----------------------------------------------------------------
C
      IPC=IPASS                                                       
C
      DO J=JDIM1,JDIM2
      DO I=IDIM1,IDIM2
        Z(I,J)=0.
      ENDDO
      ENDDO
C***
C***  FILTER THE INTERIOR POINTS WITH 25-PT BLECK FILTER               
C***
      DO 105 IP=1,IPC                                                 
C
!$omp parallel do private(htot,i,im1,im2,im3,im4,ip1,ip2,ip3,ip4,
!$omp&                    j,jm1,jm2,jm3,jm4,jp1,jp2,jp3,jp4,
!$omp&                    rsumh,sumh)
      DO 102 J=MYJS4,MYJE4
      DO 101 I=MYIS2,MYIE2
C
      IIVE=I+IVE(J)
      IIVW=I+IVW(J)
      HTOT=TM(I,J)                              
     1    +TM(IIVW,J-1)+TM(IIVE,J-1)+TM(IIVE,J+1)+TM(IIVW,J+1)
     2    +TM(I-1,J-2)+TM(I+1,J-2)+TM(I+1,J+2)+TM(I-1,J+2)
     3    +TM(I,J-2)+TM(I,J+2)+TM(I+1,J)+TM(I-1,J)
     4    +TM(IIVE,J-3)+TM(IIVE+1,J-1)+TM(IIVW,J-3)+TM(IIVE+1,J+1) 
     5    +TM(IIVW-1,J-1)+TM(IIVE,J+3)+TM(IIVW-1,J+1)+TM(IIVW,J+3)
     6    +TM(I,J-4)+TM(I+2,J)+TM(I-2,J)+TM(I,J+4)
C
      IF(HTOT.GT.0.)THEN
        SUMH=CF1*TM(I,J)                              
     1   +CF2*(TM(IIVW,J-1)+TM(IIVE,J-1)+TM(IIVE,J+1)+TM(IIVW,J+1))
     2   +CF3*(TM(I-1,J-2)+TM(I+1,J-2)+TM(I+1,J+2)+TM(I-1,J+2))
     3   +CF4*(TM(I,J-2)+TM(I,J+2)+TM(I+1,J)+TM(I-1,J))
     4   +CF5*(TM(IIVE,J-3)+TM(IIVE+1,J-1)+TM(IIVW,J-3)+TM(IIVE+1,J+1) 
     5        +TM(IIVW-1,J-1)+TM(IIVE,J+3)+TM(IIVW-1,J+1)+TM(IIVW,J+3))
     6   +CF6*(TM(I,J-4)+TM(I+2,J)+TM(I-2,J)+TM(I,J+4))
C
        RSUMH=1./SUMH
C
      Z(I,J)=(CF1*ZI(I,J)*TM(I,J)                              
     1  +CF2*(ZI(IIVW,J-1)*TM(IIVW,J-1)+ZI(IIVE,J-1)*TM(IIVE,J-1)
     2       +ZI(IIVE,J+1)*TM(IIVE,J+1)+ZI(IIVW,J+1)*TM(IIVW,J+1))
     3  +CF3*(ZI(I-1,J-2)*TM(I-1,J-2)+ZI(I+1,J-2)*TM(I+1,J-2)
     4       +ZI(I+1,J+2)*TM(I+1,J+2)+ZI(I-1,J+2)*TM(I-1,J+2))
     5  +CF4*(ZI(I,J-2)*TM(I,J-2)+ZI(I,J+2)*TM(I,J+2)
     6       +ZI(I+1,J)*TM(I+1,J)+ZI(I-1,J)*TM(I-1,J))
     7  +CF5*(ZI(IIVE,J-3)*TM(IIVE,J-3)+ZI(IIVE+1,J-1)*TM(IIVE+1,J-1)
     8       +ZI(IIVW,J-3)*TM(IIVW,J-3)+ZI(IIVE+1,J+1)*TM(IIVE+1,J+1) 
     9       +ZI(IIVW-1,J-1)*TM(IIVW-1,J-1)+ZI(IIVE,J+3)*TM(IIVE,J+3)
     o       +ZI(IIVW-1,J+1)*TM(IIVW-1,J+1)+ZI(IIVW,J+3)*TM(IIVW,J+3))
     1  +CF6*(ZI(I,J-4)*TM(I,J-4)+ZI(I+2,J)*TM(I+2,J)
     2       +ZI(I-2,J)*TM(I-2,J)+ZI(I,J+4)*TM(I,J+4)))
     3  *RSUMH
      ENDIF
C
  101 CONTINUE                                                    
  102 CONTINUE                                                       
C
!$omp parallel do 
      DO J=MYJS4,MYJE4
      DO I=MYIS2,MYIE2
        IF(TM(I,J).GT.0.5)THEN
          ZI(I,J)=Z(I,J)                                         
        ENDIF
      ENDDO
      ENDDO
C
      CALL EXCH(ZI,1,4,4)
C
  105 CONTINUE                                                          
C
      RETURN                                                            
      END                                                               
