C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& 
                            SUBROUTINE HAD2_DRY
C     ****************************************************************** 
C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    . 
C SUBPROGRAM:    HAD2_DRY    HORIZONTAL ADVECTION OF TKE
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 96-07-19 
C 
C ABSTRACT: 
C     HAD2_DRY CALCULATES THE CONTRIBUTION OF THE HORIZONTAL
C     ADVECTION TO THE TENDENCY OF TKE AND UPDATES THAT VARIABLE.
C     AN ANTI-FILTERING TECHNIQUE IS USED. 
C 
C PROGRAM HISTORY LOG: 
C   96-07-19  JANJIC  - ORIGINATOR 
C   96-08-    BLACK   - CRAY DIRECTIVES, CRAY OPTIMIZATION 
C   99-07-    JANJIC  - ADDED HORIZONTAL ADVECTION OF TKE 
C   99-07-    JANJIC  - ADDED VERTICAL ADVECTION 
C   00-01-06  BLACK   - DISTRIBUTED MEMORY AND THREADS
C   01-09-15  JANJIC  - DXDY, HORIZONTAL ADVECTION LOOPING
C   02-04-05  BLACK   - REMOVED WATER FROM HAD2
C 
C USAGE: CALL HAD2_DRY FROM SUBROUTINE DIGFLT
C   INPUT ARGUMENT LIST: 
C       NONE 
C 
C   OUTPUT ARGUMENT LIST 
C       NONE 
C 
C   OUTPUT FILES: 
C       NONE 
C   SUBPROGRAMS CALLED: 
C 
C     UNIQUE: NONE 
C 
C     LIBRARY: NONE 
C 
C   COMMON BLOCKS: CTLBLK 
C                  LOOPS 
C                  MASKS 
C                  DYNAM 
C                  CONTIN 
C                  VRBLS 
C                  PVRBLS 
C                  INDX 
C 
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN 90 
C   MACHINE : IBM SP
C$$$ 
C*********************************************************************** 
      INCLUDE "parmeta" 
      INCLUDE "parm.tbl"
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(IM1=IM-1,JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3,LP1=LM+1) 
C----------------------------------------------------------------------- 
                             P A R A M E T E R
     & (EPSQ=1.E-12,EPSQ2=0.2,CLIMIT=1.E-20
!zj     &,FF1=0.52500,FF2=-0.64813,FF3=0.24520,FF4=-0.12189) 
     &,FF1=0.53000,FF2=-0.64813,FF3=0.24520,FF4=-0.12189)
C----------------------------------------------------------------------- 
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA
C----------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "DYNAM.comm" 
!-----------------------------------------------------------------------
      include "PHYS2.comm"
C----------------------------------------------------------------------- 
      INCLUDE "CONTIN.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
!-----------------------------------------------------------------------
      include 'NHYDRO.comm'
!----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C
C-------------LOCAL VARIABLES REQUIRING DECLARATION--------------------- 
                             D I M E N S I O N
     & IFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,IFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,AFP (IDIM1:IDIM2,JDIM1:JDIM2,LM),AFQ (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DEST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E2  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,DARE(IDIM1:IDIM2,JDIM1:JDIM2),   DVOL(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,EMH (IDIM1:IDIM2,JDIM1:JDIM2)
!zj     &,hmask(idim1:idim2,jdim1:jdim2),hmask1(idim1:idim2,jdim1:jdim2)
!----------------------------------------------------------------------- 
                             R E A L
     & GSUMS(2,LM),XSUMS(2,LM)
C-----------------------------------------------------------------------
C--------------HORIZONTAL ADVECTION------------------------------------- 
!-----------------------------------------------------------------------
!zj dy in common dynam
!
      rdy=1./dy
      slopac=slopht*sqrt(2.)*0.5*50.
      crit=slopac*float(idtad)*dt*rdy*1000.
!----------------------------------------------------------------------- 
      ADDT=FLOAT(IDTAD)*DT
C***********************************************************************
      ENH=ADDT/(08.*DY)
!-----------------------------------------------------------------------
!$omp parallel do
      DO J=MYJS_P3,MYJE_P3
      DO I=MYIS_P2,MYIE_P2
        EMH (I,J)=ADDT/(08.*DX(I,J))
        dare(i,j)=hbm3(i,j)*dx(i,j)*dy
        E1(I,J,1)=MAX(Q2(I,J,1)*0.5,EPSQ2)
        E2(I,J,1)=E1(I,J,1)
      ENDDO
      ENDDO
C*********************************************************************** 
!$omp parallel do
!$omp& private (destij,dvolp,hbm2ij,hm,
!$omp&          jfp,jfq,pp,qp,sumne,sumpe,
!$omp&          tta,ttb)
      DO 300 L=1,LM
C*********************************************************************** 
      DO J=MYJS_P3,MYJE_P3
      DO I=MYIS_P2,MYIE_P2
        dvol(i,j,l)=dare(i,j)*(deta1(l)*pdtop+deta2(l)*pdsl(i,j)) 
      ENDDO 
      ENDDO 
C
      IF(L.GT.1)THEN 
        DO J=MYJS_P3,MYJE_P3
        DO I=MYIS_P2,MYIE_P2
          E1(I,J,L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
          E2(I,J,L)=E1(I,J,L) 
        ENDDO 
        ENDDO 
      ENDIF 
C----------------------------------------------------------------------- 
      DO 225 J=MYJS2_P1,MYJE2_P1
      DO 225 I=MYIS1_P1,MYIE1_P1
!
      hm=htm(i,j,l)*hbm2(i,j)
!
      tta=(u(i,j-1,l)+u(i+ihw(j),j,l)+u(i+ihe(j),j,l)+u(i,j+1,l)) 
     2   *emh(i,j)*hm 
      ttb=(v(i,j-1,l)+v(i+ihw(j),j,l)+v(i+ihe(j),j,l)+v(i,j+1,l)) 
     2   *enh*hm 
! 
      spp=-tta-ttb 
      sqp= tta-ttb 
!
          if(spp.lt.0.) then
      jfp=-1
          else
      jfp=1
          endif
!
          if(sqp.lt.0.) then 
      jfq=-1
          else
      jfq=1
          endif 
! 
      IFPA(I,J,L)=IHE(J)+I+( JFP-1)/2 
      IFQA(I,J,L)=IHE(J)+I+(-JFQ-1)/2 
! 
      JFPA(I,J,L)=J+JFP 
      JFQA(I,J,L)=J+JFQ 
! 
      IFPF(I,J,L)=IHE(J)+I+(-JFP-1)/2 
      IFQF(I,J,L)=IHE(J)+I+( JFQ-1)/2 
! 
      JFPF(I,J,L)=J-JFP 
      JFQF(I,J,L)=J-JFQ 
!-----------------------------------------------------------------------
                 if(.not.hydro) then ! z currently not available for hydro=.true.
      dza=(z(ifpa(i,j,l),jfpa(i,j,l),l)-z(i,j,l))*rdy
      dzb=(z(ifqa(i,j,l),jfqa(i,j,l),l)-z(i,j,l))*rdy
              if(abs(dza).gt.slopac) then
          ssa=dza*spp
          if(ssa.gt.crit) then
!zj      spp=crit/dza
      spp=0. !spp*.1
          endif
              endif
              if(abs(dzb).gt.slopac) then
          ssb=dzb*sqp
          if(ssb.gt.crit) then
!zj      sqp=crit/dzb
      sqp=0. !sqp*.1
          endif
              endif
                  endif
!-----------------------------------------------------------------------
      spp=spp*htm(ifpa(i,j,l),jfpa(i,j,l),l) 
      sqp=sqp*htm(ifqa(i,j,l),jfqa(i,j,l),l) 
!
      fpq=spp*sqp*htm(i,j-2,l)*htm(i-1,j,l)
     &           *htm(i+1,j,l)*htm(i,j+2,l)*0.25
!
      pp=abs(spp)
      qp=abs(sqp)
C 
      AFP (I,J,L)=(((FF4*PP+FF3)*PP+FF2)*PP+FF1)*PP 
      AFQ (I,J,L)=(((FF4*QP+FF3)*QP+FF2)*QP+FF1)*QP 
!
      e2(i,j,l)=(e1 (ifpa(i,j,l),jfpa(i,j,l),l)-e1 (i,j,l))*pp 
     &         +(e1 (ifqa(i,j,l),jfqa(i,j,l),l)-e1 (i,j,l))*qp 
     &         +(e1 (i,j-2,l)+e1 (i,j+2,l)-e1 (i-1,j,l)-e1 (i+1,j,l))
     &         *fpq
     &         +e1(i,j,l)
!
  225 continue
!-----------------------------------------------------------------------
  300 CONTINUE          ! End of Vertical Loop
C--------------ANTI-FILTERING STEP-------------------------------------- 
      do 400 l=1,lm
!-----------------------------------------------------------------------
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 350 J=MYJS2,MYJE2
      DO 350 I=MYIS1,MYIE1
!
      DVOLP=DVOL(I,J,L)
      E2IJ =E2(I,J,L)
!
      hafp=htm(ifpf(i,j,l),jfpf(i,j,l),l)*afp(i,j,l) !zj*hmask(i,j)
      hafq=htm(ifqf(i,j,l),jfqf(i,j,l),l)*afq(i,j,l) !zj*hmask(i,j)
!
      d2pqe=(e2(ifpa(i,j,l),jfpa(i,j,l),l)-e2ij
     2      -e2ij+e2(ifpf(i,j,l),jfpf(i,j,l),l))
     3      *hafp
     4     +(e2(ifqa(i,j,l),jfqa(i,j,l),l)-e2ij
     5      -e2ij+e2(ifqf(i,j,l),jfqf(i,j,l),l))
     6      *hafq
!
!zj      if(d2pqq.gt.0.) d2pqq=d2pqq*1.05
!zj      if(d2pqw.gt.0.) d2pqw=d2pqw*1.05
! 
      ESTIJ=E2IJ-D2PQE 
C 
      E00=E1 (I          ,J          ,L) 
      EP0=E1 (IFPA(I,J,L),JFPA(I,J,L),L) 
      E0Q=E1 (IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      ESTIJ=MAX(ESTIJ,MIN(E00,EP0,E0Q)) 
      ESTIJ=MIN(ESTIJ,MAX(E00,EP0,E0Q)) 
C 
      destij=estij-e1(i,j,l)
C 
      DEST(I,J,L)=DESTIJ 
C 
      DESTIJ=DESTIJ*DVOLP 
C 
      IF(DESTIJ.GT.0.)THEN
        SUMPE=SUMPE+DESTIJ
      ELSE
        SUMNE=SUMNE+DESTIJ
      ENDIF
C
  350 CONTINUE
C-----------------------------------------------------------------------
      XSUMS(1,L)=SUMPE
      XSUMS(2,L)=SUMNE
C
  400 CONTINUE            ! End of Vertical Loop
C-----------------------------------------------------------------------
C
C***  GLOBAL REDUCTION
C
      CALL MPI_ALLREDUCE(XSUMS,GSUMS,2*LM,MPI_REAL,MPI_SUM,
     1                     MPI_COMM_COMP,IRECV)
      IF(IRECV.NE.0)THEN
         PRINT*, ' RETURN CODE FROM 2nd ALLREDUCE IN ADV2 = ',IRECV
         STOP
      ENDIF
C
C***  END OF GLOBAL REDUCTION
C
C-----------------------------------------------------------------------
      DO 425 L=1,LM
C
      SUMPE=GSUMS(1,L)
      SUMNE=GSUMS(2,L)
C
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
C
      IF(SUMPE.GT.1.)THEN
        RFACE=-SUMNE/SUMPE
      ELSE
        RFACE=1.
      ENDIF
!-----------------------------------------------------------------------
!      rface=1.
!-----------------------------------------------------------------------
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1)RFACE=1. 
C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
      IF(RFACE.LT.1.)THEN
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.GE.0.)DESTIJ=DESTIJ*RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.LT.0.)DESTIJ=DESTIJ/RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
C***
      IF(L.EQ.LM)GO TO 425
C***
      IF(L.EQ.1)THEN 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-EPSQ2,EPSQ2)*HTM(I,J,1) 
        ENDDO 
        ENDDO 
      ELSE 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          IF(L.LT.LMH(I,J))THEN 
            Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-Q2(I,J,L-1),EPSQ2)
     1                   *HTM(I,J,L) 
          ELSE
            Q2(I,J,L)=Q2(I,J,L-1)
          ENDIF 
        ENDDO 
        ENDDO 
      ENDIF 
C*********************************************************************** 
  425 CONTINUE
C----------------------------------------------------------------------- 
                             RETURN 
                             END 
