                             SUBROUTINE PFDHT
C     ******************************************************************
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .     
C SUBPROGRAM:    PFDHT       PRESSURE GRADIENT/CORIOLIS CALC
C   PRGRMMR: JANJIC          ORG: W/NMC2     DATE: 93-10-28       
C     
C ABSTRACT:
C     PFDHT CALCULATES THE PRESSURE GRADIENT FORCE, UPDATES THE
C     VELOCITY COMPONENTS DUE TO THE EFFECT OF THE PRESSURE GRADIENT
C     AND CORIOILS FORCES, COMPUTES THE DIVERGENCE INCLUDING THE
C     MODIFICATION PREVENTING GRAVITY WAVE GRID SEPARATION, AND
C     CALCULATES THE HORIZONTAL PART OF THE OMEGA-ALPHA TERM
C     (THE PART PROPORTIONAL TO THE ADVECTION OF MASS ALONG
C     ETA SURFACES).
C     
C PROGRAM HISTORY LOG:
C   85-07-??  JANJIC     - ORIGINATOR
C   95-03-25  BLACK      - CONVERSION FROM 1-D TO 2-D IN HORIZONTAL
C   96-05-??  JANJIC     - NONHYDROSTATIC CONVERSION
C   00-01-06  BLACK      - DISTRIBUTED MEMORY AND THREADS
C     
C USAGE: CALL PFDHT FROM MAIN PROGRAM EBU
C   INPUT ARGUMENT LIST:
C       NONE     
C  
C   OUTPUT ARGUMENT LIST: 
C     NONE
C     
C   OUTPUT FILES:
C     NONE
C     
C   SUBPROGRAMS CALLED:
C  
C     UNIQUE: NONE
C  
C     LIBRARY: NONE
C  
C   COMMON BLOCKS: CTLBLK
C                  MASKS
C                  DYNAM
C                  SCRCH
C                  VRBLS
C                  NHYDRO
C                  INDX
C   
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 90
C   MACHINE : IBM SP
C$$$  
C***********************************************************************
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      INCLUDE "parmeta"
      INCLUDE "mpp.h"
#include "sp.h"
C-----------------------------------------------------------------------
                             P A R A M E T E R
     & (LP1=LM+1,JAM=6+2*(JM-10))
C-----------------------------------------------------------------------
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA
C----------------------------------------------------------------------
      INCLUDE "CTLBLK.comm"
c-----------------------------------------------------------------------
      include "LOOPS.comm"
C-----------------------------------------------------------------------
      INCLUDE "MASKS.comm"
C-----------------------------------------------------------------------
      INCLUDE "INDX.comm"
c-----------------------------------------------------------------------
      INCLUDE "DYNAM.comm"
C-----------------------------------------------------------------------
      INCLUDE "VRBLS.comm"
c-----------------------------------------------------------------------
      INCLUDE "NHYDRO.comm"
c-----------------------------------------------------------------------
      INCLUDE "CONTIN.comm"
c-----------------------------------------------------------------------
      INCLUDE "CLDWTR.comm"
C-----------------------------------------------------------------------
                             R E A L
     & FILO  (IDIM1:IDIM2,JDIM1:JDIM2),fim   (idim1:idim2,jdim1:jdim2)
     &,RDPD  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ADPDX (IDIM1:IDIM2,JDIM1:JDIM2),RDPDX (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ADPDY (IDIM1:IDIM2,JDIM1:JDIM2),RDPDY (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ADPDNE(IDIM1:IDIM2,JDIM1:JDIM2),ADPDSE(IDIM1:IDIM2,JDIM1:JDIM2)
     &,UDY   (IDIM1:IDIM2,JDIM1:JDIM2),VDX   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PEW   (IDIM1:IDIM2,JDIM1:JDIM2),PNS   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PCEW  (IDIM1:IDIM2,JDIM1:JDIM2),PCNS  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,DPFEW (IDIM1:IDIM2,JDIM1:JDIM2),DPFNS (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TNS   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,HM    (IDIM1:IDIM2,JDIM1:JDIM2)
C
                             R E A L
     & DPDE  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,APEL  (IDIM1:IDIM2,JDIM1:JDIM2),PCXC  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,DFDZ  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TEW   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TNE   (IDIM1:IDIM2,JDIM1:JDIM2),TSE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PNE   (IDIM1:IDIM2,JDIM1:JDIM2),PSE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,CNE   (IDIM1:IDIM2,JDIM1:JDIM2),CSE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PPNE  (IDIM1:IDIM2,JDIM1:JDIM2),PPSE  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,PCNE  (IDIM1:IDIM2,JDIM1:JDIM2),PCSE  (IDIM1:IDIM2,JDIM1:JDIM2)
C-----------------------------------------------------------------------
C--------------PREPARATORY CALCULATIONS---------------------------------
C-----------------------------------------------------------------------
      IF(SIGMA)THEN
!$omp parallel do
!zj 4 changed to 5
        DO 50 J=MYJS_P4,MYJE_P4
        DO 50 I=MYIS_P4,MYIE_P4
        FILO(I,J)=FIS(I,J)
        PDSL(I,J)=PD(I,J)
   50   CONTINUE
      ELSE
!$omp parallel do
        DO 100 J=MYJS_P4,MYJE_P4
        DO 100 I=MYIS_P4,MYIE_P4
        FILO(I,J)=0.0
        PDSL(I,J)=RES(I,J)*PD(I,J)
  100   CONTINUE
      ENDIF
C
!$omp parallel do
      DO L=1,LM
        DO J=MYJS_P4,MYJE_P4
        DO I=MYIS_P4,MYIE_P4
          OMGALF(I,J,L)=0.
          DIV(I,J,L)=0.
        ENDDO
        ENDDO
      ENDDO
C-------------------- MAIN VERTICAL INTEGRATION LOOP -------------------
!      CALL ZERO2(DPDE)
!      CALL ZERO2(APEL)
!      CALL ZERO2(PCXC)
!      CALL ZERO2(DFDZ)
!      CALL ZERO2(UDY)
!      CALL ZERO2(VDX)
!      CALL ZERO2(TEW)
!      CALL ZERO2(TNE)
!      CALL ZERO2(TSE)
!      CALL ZERO2(PNE)
!      CALL ZERO2(PSE)
!      CALL ZERO2(CNE)
!      CALL ZERO2(CSE)
!      CALL ZERO2(PPNE)
!      CALL ZERO2(PPSE)
!      CALL ZERO2(PCNE)
!      CALL ZERO2(PCSE)
!      call zero2(fim)
!
!zj      call zero3(few)
!zj      call zero3(fns)
!zj      call zero3(fne)
!zj      call zero3(fse)
C-----------------------------------------------------------------------
!$omp parallel do
!$omp& private(adpdne,adpdse,adpdx,adpdy,
!$omp&         apel,cne,cse,dcnek,dcsek,dfdz,dpde,dpfew,dpfnek,
!$omp&         dpfns,dpfsek,dpnek,few,fne,fns,fse,hm,
!$omp&         pcew,pcne,pcns,pcse,pcxc,pew,pne,pns,ppne,ppse,
!$omp&         pse,pvnek,pvsek,rdpd,rdpdx,rdpdy,tew,tne,tns,tse,
!$omp&         udy,vdx,vm)
C-----------------------------------------------------------------------
                             do 400 l=lm,1,-1
C-----------------------------------------------------------------------
!      CALL ZERO2(DPDE)
!      CALL ZERO2(APEL)
!      CALL ZERO2(PCXC)
!      CALL ZERO2(DFDZ)
!      CALL ZERO2(UDY)
!      CALL ZERO2(VDX)
!      CALL ZERO2(TEW)
!      CALL ZERO2(TNE)
!      CALL ZERO2(TSE)
!      CALL ZERO2(PNE)
!      CALL ZERO2(PSE)
!      CALL ZERO2(CNE)
!      CALL ZERO2(CSE)
!      CALL ZERO2(PPNE)
!      CALL ZERO2(PPSE)
!      CALL ZERO2(PCNE)
!      CALL ZERO2(PCSE)
!      call zero2(fim)
C-----------------------------------------------------------------------
      DO 205 J=MYJS_P4,MYJE_P4
      DO 205 I=MYIS_P4,MYIE_P4
!
      HM(I,J)=HTM(I,J,L)*HBM2(I,J)
!
      apelp=(pint(i,j,l)+pint(i,j,l+1))*0.5
      rtopp=(q(i,j,l)*0.608-cwm(i,j,l)+1.)*t(i,j,l)*r/apelp
      dfi=rtopp*(deta1(l)*pdtop+deta2(l)*pdsl(i,j))
!
      apel(i,j)=apelp
      rtop(i,j,l)=rtopp
      dfdz(i,j)=rtopp
!
      fiupp=filo(i,j)+dfi
!
      fim(i,j)=filo(i,j)+fiupp
!
      filo(i,j)=(fiupp-dfl(l))*htm(i,j,l)+dfl(l)
  205 CONTINUE
C
      DO 210 J=MYJS_P4,MYJE_P4
      DO 210 I=MYIS_P4,MYIE_P4
      dpde(i,j)=deta1(l)*pdtop+deta2(l)*pdsl(i,j)
  210 CONTINUE
C
      DO 215 J=MYJS,MYJE
      DO 215 I=MYIS,MYIE
      RDPD(I,J)=1./DPDE(I,J)
  215 CONTINUE
C
      DO 220 J=MYJS1_P3,MYJE1_P3
      DO 220 I=MYIS_P3,MYIE_P3
      ADPDX(I,J)=DPDE(I+IVW(J),J)+DPDE(I+IVE(J),J)
      ADPDY(I,J)=DPDE(I,J-1)+DPDE(I,J+1)
      RDPDX(I,J)=1./ADPDX(I,J)
      RDPDY(I,J)=1./ADPDY(I,J)
  220 CONTINUE
C
C--------------DIAGONAL CONTRIBUTIONS TO PRESSURE GRADIENT FORCE--------
C
      DO 240 J=MYJS_P3,MYJE1_P3
      DO 240 I=MYIS_P3,MYIE_P3
      ADPDNE(I,J)=DPDE(I+IHE(J),J+1)+DPDE(I,J)
      PNE(I,J)=(FIM (I+IHE(J),J+1)-FIM (I,J))
     1        *(DWDT(I+IHE(J),J+1,L)+DWDT(I,J,L))
      PPNE(I,J)=PNE(I,J)*ADPDNE(I,J)
      CNE(I,J)=(DFDZ(I+IHE(J),J+1)+DFDZ(I,J))*2.
     1        *(APEL(I+IHE(J),J+1)-APEL(I,J))
      PCNE(I,J)=CNE(I,J)*ADPDNE(I,J)
  240 CONTINUE
C
      DO 250 J=MYJS1_P3,MYJE_P3
      DO 250 I=MYIS_P3,MYIE_P3
      ADPDSE(I,J)=DPDE(I+IHE(J),J-1)+DPDE(I,J)
      PSE(I,J)=(FIM (I+IHE(J),J-1)-FIM (I,J))
     1        *(DWDT(I+IHE(J),J-1,L)+DWDT(I,J,L))
      PPSE(I,J)=PSE(I,J)*ADPDSE(I,J)
      CSE(I,J)=(DFDZ(I+IHE(J),J-1)+DFDZ(I,J))*2.
     1        *(APEL(I+IHE(J),J-1)-APEL(I,J))
      PCSE(I,J)=CSE(I,J)*ADPDSE(I,J)
  250 CONTINUE
C
C--------------CONTINUITY EQUATION MODIFICATION-------------------------
C
      DO 260 J=MYJS1_P1,MYJE1_P1
      DO 260 I=MYIS_P1,MYIE_P1
      PCXC(I,J)=VBM3(I,J)*VTM(I,J,L)*(PNE(I+IVW(J),J)
     1         +CNE(I+IVW(J),J)+PSE(I+IVW(J),J)+CSE(I+IVW(J),J)
     2         -PNE(I,J-1)-CNE(I,J-1)-PSE(I,J+1)-CSE(I,J+1))
  260 CONTINUE
C-----------------------------------------------------------------------
      DO 270 J=MYJS2,MYJE2
      DO 270 I=MYIS1,MYIE1
      div(i,j,l)=(deta1(l)*pdtop/(101300.-pt)
     &           +deta2(l)*(1.-pdtop/(101300.-pt)))
     &         *wpdar(i,j)
     1         *(PCXC(I+IHE(J),J)-PCXC(I,J+1)
     2          +PCXC(I+IHW(J),J)-PCXC(I,J-1))
  270 CONTINUE
C
C--------------LAT & LONG PRESSURE FORCE COMPONENTS---------------------
C
      DO 280 J=MYJS1_P2,MYJE1_P2
      DO 280 I=MYIS_P2,MYIE_P3
      DPNEK=PNE(I+IVW(J),J)+PNE(I,J-1)
      DPSEK=PSE(I+IVW(J),J)+PSE(I,J+1)
      PEW(I,J)=DPNEK+DPSEK
      PNS(I,J)=DPNEK-DPSEK
      DCNEK=CNE(I+IVW(J),J)+CNE(I,J-1)
      DCSEK=CSE(I+IVW(J),J)+CSE(I,J+1)
      PCEW(I,J)=(DCNEK+DCSEK)*ADPDX(I,J)
      PCNS(I,J)=(DCNEK-DCSEK)*ADPDY(I,J)
  280 CONTINUE
C
C--------------FIRST PASS SWITCH----------------------------------------
      IF(FIRST)GO TO 305
C
C--------------UPDATE U AND V PGF---------------------------------------
C
      DO 290 J=MYJS2_P2,MYJE2_P2
      DO 290 I=MYIS_P2,MYIE1_P2
      DPFNEK=((PPNE(I+IVW(J),J)+PPNE(I,J-1))
     1       +(PCNE(I+IVW(J),J)+PCNE(I,J-1)))
      dpfnek=dpfnek+dpfnek
      DPFSEK=((PPSE(I+IVW(J),J)+PPSE(I,J+1))
     1       +(PCSE(I+IVW(J),J)+PCSE(I,J+1)))
      dpfsek=dpfsek+dpfsek
      DPFEW(I,J)=DPFNEK+DPFSEK
      DPFNS(I,J)=DPFNEK-DPFSEK
  290 CONTINUE
C-----------------------------------------------------------------------
      DO 300 J=MYJS2_P3,MYJE2_P3
      DO 300 I=MYIS_P2,MYIE1_P2
        VM=VTM(I,J,L)*VBM2(I,J)
        U(I,J,L)=(((DPFEW(I,J)+PCEW(I,J))*RDPDX(I,J)
     1             +PEW(I,J))*CPGFU(I,J))*VM+U(I,J,L)
        V(I,J,L)=(((DPFNS(I,J)+PCNS(I,J))*RDPDY(I,J)
     1             +PNS(I,J))*CPGFV)*VM+V(I,J,L)
  300 CONTINUE
C
C--------------LAST PASS SWITCH-----------------------------------------
      IF(NTSD.EQ.NTSTM)GO TO 400
C-----------------------------------------------------------------------
 305  CONTINUE                                                          
C
C--------------LAT & LON FLUXES & OMEGA-ALPHA COMPONENTS----------------
C
      DO 310 J=MYJS1_P2,MYJE1_P2
      DO 310 I=MYIS_P2,MYIE_P3
      UDY(I,J)=DY*U(I,J,L)
      few(i,j,l)=udy(i,j)*adpdx(i,j)
      TEW(I,J)=UDY(I,J)*PCEW(I,J)
      VDX(I,J)=DX(I,J)*V(I,J,L)
      fns(i,j,l)=vdx(i,j)*adpdy(i,j)
      TNS(I,J)=VDX(I,J)*PCNS(I,J)
  310 CONTINUE
C
C--------------DIAGONAL FLUXES AND DIAGONALLY AVERAGED WIND-------------
C
      DO 320 J=MYJS1_P1,MYJE2_P1
      DO 320 I=MYIS_P1,MYIE1_P1
      PVNEK=(UDY(I+IHE(J),J)+VDX(I+IHE(J),J))+(UDY(I,J+1)+VDX(I,J+1))
      fne(i,j,l)=pvnek*adpdne(i,j)
      TNE(I,J)=PVNEK*PCNE(I,J)*2.
  320 CONTINUE
C
      DO 330 J=MYJS2_P1,MYJE1_P1
      DO 330 I=MYIS_P1,MYIE1_P1
      PVSEK=(UDY(I+IHE(J),J)-VDX(I+IHE(J),J))+(UDY(I,J-1)-VDX(I,J-1))
      fse(i,j,l)=pvsek*adpdse(i,j)
      TSE(I,J)=PVSEK*PCSE(I,J)*2.
  330 CONTINUE
C
C--------------HORIZONTAL PART OF OMEGA-ALPHA & DIVERGENCE--------------
C
      DO 340 J=MYJS2,MYJE2
      DO 340 I=MYIS1,MYIE1


!zj              if(mype.eq.0.and.l.eq.lm.and.i.eq.10.and.j.eq.10) then
!zj          print*,'pfdht2',ntsd,i,j,tew(i,j),tns(i,j),tne(i,j),tse(i,j),
!zj     &  rdpd(i,j),fcp(i,j),hm(i,j),omgalf(i,j,l),t(i,j,l)
!zj              endif



      OMGALF(I,J,L)=(TEW(I+IHE(J),J)+TEW(I+IHW(J),J)+TNS(I,J+1)
     1              +TNS(I,J-1)+TNE(I,J)+TNE(I+IHW(J),J-1)+TSE(I,J)
     2              +TSE(I+IHW(J),J+1))*RDPD(I,J)*FCP(I,J)*HM(I,J)



!      print*,i,j,l
!     &,TEW(I+IHE(J),J),TEW(I+IHW(J),J),TNS(I,J+1)
!     1,TNS(I,J-1),TNE(I,J),TNE(I+IHW(J),J-1),TSE(I,J)
!     2,TSE(I+IHW(J),J+1)

      T(I,J,L)=OMGALF(I,J,L)+T(I,J,L)
      div(i,j,l)=(((few(i+ihe(j),j,l)+fns(i,j+1,l)
     1             +fne(i,j,l)+fse(i,j,l))
     2            -(few(i+ihw(j),j,l)+fns(i,j-1,l)
     3             +fne(i+ihw(j),j-1,l)+fse(i+ihw(j),j+1,l)))*fdiv(i,j)
     5           +div(i,j,l))*hm(i,j)

!zj              if(mype.eq.0.and.l.eq.lm.and.i.eq.10.and.j.eq.10) then
!zj          print*,'pfdht2-after',ntsd,i,j,omgalf(i,j,l),t(i,j,l)
!zj              endif



  340 CONTINUE
!-----------------------------------------------------------------------
  400                        CONTINUE
C------------------------------------------------------------------------
                             RETURN
                             END                                        
