C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& 
                            SUBROUTINE VAD2 
C     ****************************************************************** 
C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    . 
C SUBPROGRAM:    ADV2        HORIZ/VERT ADVECTION OF VAPOR AND CLOUD 
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 96-07-19 
C 
C ABSTRACT: 
C     ADV2 CALCULATES THE CONTRIBUTION OF THE HORIZONTAL AND VERTICAL
C     ADVECTION TO THE TENDENCIES OF WATER SUBSTANCE AND TKE AND THEN 
C     UPDATES THOSE VARIABLES.  AN ANTI-FILTERING TECHNIQUE IS USED. 
C 
C PROGRAM HISTORY LOG: 
C   96-07-19  JANJIC  - ORIGINATOR 
C   96-08-    BLACK   - CRAY DIRECTIVES, CRAY OPTIMIZATION 
C   99-07-    JANJIC  - ADDED HORIZONTAL ADVECTION OF TKE 
C   99-07-    JANJIC  - ADDED VERTICAL ADVECTION 
C   00-01-06  BLACK   - DISTRIBUTED MEMORY AND THREADS
!   01-09-15  JANJIC  - DXDY, HORIZONTAL ADVECTION LOOPING
C 
C USAGE: CALL ADV2 FROM MAIN PROGRAM 
C   INPUT ARGUMENT LIST: 
C       NONE 
C 
C   OUTPUT ARGUMENT LIST 
C       NONE 
C 
C   OUTPUT FILES: 
C       NONE 
C   SUBPROGRAMS CALLED: 
C 
C     UNIQUE: NONE 
C 
C     LIBRARY: NONE 
C 
C   COMMON BLOCKS: CTLBLK 
C                  LOOPS 
C                  MASKS 
C                  DYNAM 
C                  CONTIN 
C                  VRBLS 
C                  PVRBLS 
C                  CLDWTR 
C                  INDX 
C 
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN 90 
C   MACHINE : IBM SP
C$$$ 
C*********************************************************************** 
      INCLUDE "parmeta" 
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(IM1=IM-1,JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3,LP1=LM+1) 
C----------------------------------------------------------------------- 
                             P A R A M E T E R
     &(EPSQ=1.E-12,EPSQ2=0.2,CLIMIT=1.E-20
     &,FF1=0.52500,FF2=-0.64813,FF3=0.24520,FF4=-0.12189) 
C----------------------------------------------------------------------- 
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA,TOP,BOT 
C----------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "DYNAM.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CONTIN.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CLDWTR.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C
C-------------LOCAL VARIABLES REQUIRING DECLARATION--------------------- 
C
                             D I M E N S I O N
     & Q3(LM),W3(LM),E3(LM),Q4(LM),W4(LM),E4(LM) 
     &,petdtl(LM),DQL(LM),DWL(LM),DEL(LM),LA(LM),AFR(LM)  
C 
C-----------------------------------------------------------------------
C*********************************************************************** 
      ADDT=FLOAT(IDTAD)*DT 
C----------------------------------------------------------------------- 
C-----------------VERTICAL ADVECTION------------------------------------ 
C----------------------------------------------------------------------- 
!$omp parallel do
!$omp& private (afr,afrp,arr,bot,d2pqe,d2pqq,d2pqw,del,dep,detap,
!$omp&          dpdn,dpup,dql,dqp,dwl,dwp,e00,e3,e4,e4p,ep,ep0,
!$omp&          hbm2ij,la,lap,llap,lmp,petdtl,q00,q3,q4,q4p,qp,qp0,
!$omp&          rface,rfacq,rfacw,rr,sumne,sumnq,sumnw,
!$omp&          sumpe,sumpq,sumpw,top,w00,w3,w4,w4p,wp,wp0)
C----------------------------------------------------------------------- 
      DO 200 J=MYJS2,MYJE2
      DO 200 I=MYIS1,MYIE1
C----------------------------------------------------------------------- 
      LMP=LMH(I,J) 
      hbm=hbm2(i,j)
C----------------------------------------------------------------------- 
      E3(1)=Q2(I,J,1)*0.5 
      DO L=2,LMP 
        E3(L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
      ENDDO 
C
      DO L=1,LMP 
        Q3(L)=MAX(Q(I,J,L),EPSQ) 
        W3(L)=MAX(CWM(I,J,L),CLIMIT) 
        E4(L)=E3(L) 
        Q4(L)=Q3(L) 
        W4(L)=W3(L) 
      ENDDO 
C 
      petdtl(1)=petdt(i,j,1)*0.5 
C
      DO L=2,LMP-1 
        petdtl(l)=(petdt(i,j,l-1)+petdt(i,j,l))*0.5 
      ENDDO 
C
      petdtl(lmp)=petdt(i,j,lmp-1)*0.5 
C----------------------------------------------------------------------- 
      haddt=-addt*hbm
      DO 25 L=1,LMP
      rr=petdtl(l)*haddt
!
          if(rr.lt.0.) then
      lap=-1
          else
      lap=1
          endif
!
      LA(L)=LAP
      LLAP=L+LAP
C
      TOP=.FALSE.
      BOT=.FALSE.
C
      IF(LLAP.GT.0.AND.LLAP.LT.LMH(I,J)+1.AND.LAP.NE.0)THEN
!
        rr=abs(rr/((aeta1(llap)-aeta1(l))*pdtop
     &            +(aeta2(llap)-aeta2(l))*pdsl(i,j)))
!
        AFR(L)=(((FF4*RR+FF3)*RR+FF2)*RR+FF1)*RR 
        DQP=(Q3(LLAP)-Q3(L))*RR 
        DWP=(W3(LLAP)-W3(L))*RR 
        DEP=(E3(LLAP)-E3(L))*RR 
        DQL(L)=DQP 
        DWL(L)=DWP            
        DEL(L)=DEP 
      ELSE 
        TOP=LLAP.EQ.0 
        BOT=LLAP.EQ.LMH(I,J)+1 
C 
        RR=0. 
        AFR(L)=0. 
        DQL(L)=0. 
        DWL(L)=0.            
        DEL(L)=0. 
      ENDIF 
   25 CONTINUE
C 
      IF(TOP)THEN 
        IF(LA(2).LT.0)THEN 
          rfc=(deta1(2)*pdtop+deta2(2)*pdsl(i,j))
     &       /(deta1(1)*pdtop+deta2(1)*pdsl(i,j))
          dql(1)=-dql(2)*rfc
          dwl(1)=-dwl(2)*rfc
          del(1)=-del(2)*rfc 
        ENDIF 
      ENDIF 
C
      IF(BOT)THEN 
        IF(LA(LMP-1).GT.0)THEN 
          rfc=(deta1(lmp-1)*pdtop+deta2(lmp-1)*pdsl(i,j))
     &       /(deta1(lmp)*pdtop+deta2(lmp)*pdsl(i,j))
          dql(lmp)=-dql(lmp-1)*rfc
          dwl(lmp)=-dwl(lmp-1)*rfc
          del(lmp)=-del(lmp-1)*rfc
        ENDIF 
      ENDIF 
C
        DO L=1,LMP
          q4(l)=q3(l)+dql(l)
          w4(l)=w3(l)+dwl(l)
          e4(l)=e3(l)+del(l)
        ENDDO
C--------------ANTI-FILTERING STEP-------------------------------------- 
      SUMPQ=0. 
      SUMNQ=0. 
      SUMPW=0. 
      SUMNW=0. 
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 50 L=2,LMP-1 
      detap=deta1(l)*pdtop+deta2(l)*pdsl(i,j) 
C 
      Q4P=Q4(L) 
      W4P=W4(L) 
      E4P=E4(L) 
C 
      LAP=LA(L) 
C 
      IF(LAP.NE.0)THEN 
        dpdn=(aeta1(l+lap)-aeta1(l))*pdtop
     &      +(aeta2(l+lap)-aeta2(l))*pdsl(i,j)
        dpup=(aeta1(l)-aeta1(l-lap))*pdtop
     &      +(aeta2(l)-aeta2(l-lap))*pdsl(i,j)
!
        afrp=2.*afr(l)*dpdn*dpdn/(dpdn+dpup)
        d2pqq=((q4(l+lap)-q4p)/dpdn 
     1        -(q4p-q4(l-lap))/dpup)*afrp 
        d2pqw=((w4(l+lap)-w4p)/dpdn 
     1        -(w4p-w4(l-lap))/dpup)*afrp 
        d2pqe=((e4(l+lap)-e4p)/dpdn 
     1        -(e4p-e4(l-lap))/dpup)*afrp 
      ELSE 
        D2PQQ=0. 
        D2PQW=0. 
        D2PQE=0. 
      ENDIF 
C 
      QP=Q4P-D2PQQ 
      WP=W4P-D2PQW 
      EP=E4P-D2PQE 
C 
      Q00=Q3(L) 
      QP0=Q3(L+LAP) 
C 
      W00=W3(L) 
      WP0=W3(L+LAP) 
C 
      E00=E3(L) 
      EP0=E3(L+LAP) 
C 
      IF(LAP.NE.0)THEN 
        QP=MAX(QP,MIN(Q00,QP0)) 
        QP=MIN(QP,MAX(Q00,QP0)) 
        WP=MAX(WP,MIN(W00,WP0)) 
        WP=MIN(WP,MAX(W00,WP0)) 
        EP=MAX(EP,MIN(E00,EP0)) 
        EP=MIN(EP,MAX(E00,EP0)) 
      ENDIF 
C 
      dqp=qp-q00 
      dwp=wp-w00 
      dep=ep-e00 
C 
      DQL(L)=DQP 
      DWL(L)=DWP 
      DEL(L)=DEP 
C 
      dqp=dqp*detap 
      dwp=dwp*detap 
      dep=dep*detap 
C 
      IF(DQP.GT.0.)THEN 
        SUMPQ=SUMPQ+DQP 
      ELSE 
        SUMNQ=SUMNQ+DQP 
      ENDIF 
C 
      IF(DWP.GT.0.)THEN 
        SUMPW=SUMPW+DWP 
      ELSE 
        SUMNW=SUMNW+DWP 
      ENDIF 
C 
      IF(DEP.GT.0.)THEN 
        SUMPE=SUMPE+DEP 
      ELSE 
        SUMNE=SUMNE+DEP 
      ENDIF 
C 
   50 CONTINUE
C----------------------------------------------------------------------- 
      DQL(1)=0. 
      DWL(1)=0. 
      DEL(1)=0. 
C 
      DQL(LMP)=0. 
      DWL(LMP)=0. 
      DEL(LMP)=0. 
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
      IF(SUMPQ.GT.1.E-9)THEN 
        RFACQ=-SUMNQ/SUMPQ 
      ELSE 
        RFACQ=1. 
      ENDIF 
C
      IF(SUMPW.GT.1.E-9)THEN 
        RFACW=-SUMNW/SUMPW 
      ELSE 
        RFACW=1. 
      ENDIF 
C
      IF(SUMPE.GT.1.E-9)THEN 
        RFACE=-SUMNE/SUMPE 
      ELSE 
        RFACE=1. 
      ENDIF 
C
      IF(RFACQ.LT.0.9.OR.RFACQ.GT.1.1) RFACQ=1. 
      IF(RFACW.LT.0.9.OR.RFACW.GT.1.1) RFACW=1. 
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1) RFACE=1. 
C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
        DO L=1,LMP 
          DQP=DQL(L) 
          IF(DQP.GE.0.) DQP=DQP*RFACQ 
          q(i,j,l)=q3(l)+dqp 
        ENDDO 
C----------------------------------------------------------------------- 
        DO L=1,LMP 
          DWP=DWL(L) 
          IF(DWP.GE.0.) DWP=DWP*RFACW 
          cwm(i,j,l)=w3(l)+dwp 
        ENDDO 
C----------------------------------------------------------------------- 
        DO L=1,LMP 
          DEP=DEL(L) 
          IF(DEP.GE.0.) DEP=DEP*RFACE 
          e3(l)=e3(l)+dep 
        ENDDO 
C
      q2(i,j,1)=max(e3(1)+e3(1)-epsq2,epsq2)*hbm 
     1         +q2(i,j,1)*(1.-hbm)
      DO L=2,LMP-1 
      q2(i,j,l)=max(e3(l)+e3(l)-q2(i,j,l-1),epsq2)*hbm
     1         +q2(i,j,l)*(1.-hbm)
      ENDDO 
C----------------------------------------------------------------------- 
  200 CONTINUE
C*********************************************************************** 
                             RETURN 
                             END 
