C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& 
                            SUBROUTINE VAD2_DRY
C     ****************************************************************** 
C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    . 
C SUBPROGRAM:    VAD2_DRY    VERTICAL ADVECTION OF TKE
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 96-07-19 
C 
C ABSTRACT: 
C     VAD2_DRY CALCULATES THE CONTRIBUTION OF THE VERTICAL ADVECTION
C     TO THE TENDENCY OF TKE AND THEN UPDATES THAT VARIABLE. 
C     AN ANTI-FILTERING TECHNIQUE IS USED. 
C 
C PROGRAM HISTORY LOG: 
C   96-07-19  JANJIC  - ORIGINATOR 
C   96-08-    BLACK   - CRAY DIRECTIVES, CRAY OPTIMIZATION 
C   99-07-    JANJIC  - ADDED HORIZONTAL ADVECTION OF TKE 
C   99-07-    JANJIC  - ADDED VERTICAL ADVECTION 
C   00-01-06  BLACK   - DISTRIBUTED MEMORY AND THREADS
C   01-09-15  JANJIC  - DXDY, HORIZONTAL ADVECTION LOOPING
C   02-04-05  BLACK   - REMOVED WATER FROM VAD2
C 
C USAGE: CALL VAD2_DRY FROM SUBROUTINE DIGFLT
C   INPUT ARGUMENT LIST: 
C       NONE 
C 
C   OUTPUT ARGUMENT LIST 
C       NONE 
C 
C   OUTPUT FILES: 
C       NONE 
C   SUBPROGRAMS CALLED: 
C 
C     UNIQUE: NONE 
C 
C     LIBRARY: NONE 
C 
C   COMMON BLOCKS: CTLBLK 
C                  LOOPS 
C                  MASKS 
C                  DYNAM 
C                  CONTIN 
C                  VRBLS 
C                  PVRBLS 
C                  INDX 
C 
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN 90 
C   MACHINE : IBM SP
C$$$ 
C*********************************************************************** 
      INCLUDE "parmeta" 
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(IM1=IM-1,JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3,LP1=LM+1) 
C----------------------------------------------------------------------- 
                             P A R A M E T E R
     &(EPSQ=1.E-12,EPSQ2=0.2,CLIMIT=1.E-20
     &,FF1=0.52500,FF2=-0.64813,FF3=0.24520,FF4=-0.12189) 
C----------------------------------------------------------------------- 
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA,TOP,BOT 
C----------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "DYNAM.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CONTIN.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C
C-------------LOCAL VARIABLES REQUIRING DECLARATION--------------------- 
C
                             D I M E N S I O N
     & E3(LM),E4(LM) 
     &,petdtl(LM),DEL(LM),LA(LM),AFR(LM)  
C 
C-----------------------------------------------------------------------
C*********************************************************************** 
      ADDT=FLOAT(IDTAD)*DT 
C----------------------------------------------------------------------- 
C-----------------VERTICAL ADVECTION------------------------------------ 
C----------------------------------------------------------------------- 
!$omp parallel do
!$omp& private (afr,afrp,arr,bot,d2pqe,del,dep,detap,
!$omp&          dpdn,dpup,e00,e3,e4,e4p,ep,ep0,
!$omp&          hbm2ij,la,lap,llap,lmp,petdtl,
!$omp&          rface,rr,sumne,
!$omp&          sumpe,top)
C----------------------------------------------------------------------- 
      DO 200 J=MYJS2,MYJE2
      DO 200 I=MYIS1,MYIE1
C----------------------------------------------------------------------- 
      LMP=LMH(I,J) 
      hbm=hbm2(i,j)
C----------------------------------------------------------------------- 
      E3(1)=Q2(I,J,1)*0.5 
      DO L=2,LMP 
        E3(L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
      ENDDO 
C
      DO L=1,LMP 
        E4(L)=E3(L) 
      ENDDO 
C 
      petdtl(1)=petdt(i,j,1)*0.5 
C
      DO L=2,LMP-1 
        petdtl(l)=(petdt(i,j,l-1)+petdt(i,j,l))*0.5 
      ENDDO 
C
      petdtl(lmp)=petdt(i,j,lmp-1)*0.5 
C----------------------------------------------------------------------- 
      haddt=-addt*hbm
      DO 25 L=1,LMP
      rr=petdtl(l)*haddt
!
          if(rr.lt.0.) then
      lap=-1
          else
      lap=1
          endif
!
      LA(L)=LAP
      LLAP=L+LAP
C
      TOP=.FALSE.
      BOT=.FALSE.
C
      IF(LLAP.GT.0.AND.LLAP.LT.LMH(I,J)+1.AND.LAP.NE.0)THEN
!
        rr=abs(rr/((aeta1(llap)-aeta1(l))*pdtop
     &            +(aeta2(llap)-aeta2(l))*pdsl(i,j)))
!
        AFR(L)=(((FF4*RR+FF3)*RR+FF2)*RR+FF1)*RR 
        DEP=(E3(LLAP)-E3(L))*RR 
        DEL(L)=DEP 
      ELSE 
        TOP=LLAP.EQ.0 
        BOT=LLAP.EQ.LMH(I,J)+1 
C 
        RR=0. 
        AFR(L)=0. 
        DEL(L)=0. 
      ENDIF 
   25 CONTINUE
C 
      IF(TOP)THEN 
        IF(LA(2).LT.0)THEN 
          rfc=(deta1(2)*pdtop+deta2(2)*pdsl(i,j))
     &       /(deta1(1)*pdtop+deta2(1)*pdsl(i,j))
          del(1)=-del(2)*rfc 
        ENDIF 
      ENDIF 
C
      IF(BOT)THEN 
        IF(LA(LMP-1).GT.0)THEN 
          rfc=(deta1(lmp-1)*pdtop+deta2(lmp-1)*pdsl(i,j))
     &       /(deta1(lmp)*pdtop+deta2(lmp)*pdsl(i,j))
          del(lmp)=-del(lmp-1)*rfc
        ENDIF 
      ENDIF 
C
        DO L=1,LMP
          e4(l)=e3(l)+del(l)
        ENDDO
C--------------ANTI-FILTERING STEP-------------------------------------- 
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 50 L=2,LMP-1 
      detap=deta1(l)*pdtop+deta2(l)*pdsl(i,j) 
C 
      E4P=E4(L) 
C 
      LAP=LA(L) 
C 
      IF(LAP.NE.0)THEN 
        dpdn=(aeta1(l+lap)-aeta1(l))*pdtop
     &      +(aeta2(l+lap)-aeta2(l))*pdsl(i,j)
        dpup=(aeta1(l)-aeta1(l-lap))*pdtop
     &      +(aeta2(l)-aeta2(l-lap))*pdsl(i,j)
!
        afrp=2.*afr(l)*dpdn*dpdn/(dpdn+dpup)
        d2pqe=((e4(l+lap)-e4p)/dpdn 
     1        -(e4p-e4(l-lap))/dpup)*afrp 
      ELSE 
        D2PQE=0. 
      ENDIF 
C 
      EP=E4P-D2PQE 
C 
      E00=E3(L) 
      EP0=E3(L+LAP) 
C 
      IF(LAP.NE.0)THEN 
        EP=MAX(EP,MIN(E00,EP0)) 
        EP=MIN(EP,MAX(E00,EP0)) 
      ENDIF 
C 
      dep=ep-e00 
C 
      DEL(L)=DEP 
C 
      dep=dep*detap 
C 
      IF(DEP.GT.0.)THEN 
        SUMPE=SUMPE+DEP 
      ELSE 
        SUMNE=SUMNE+DEP 
      ENDIF 
C 
   50 CONTINUE
C----------------------------------------------------------------------- 
      DEL(1)=0. 
C 
      DEL(LMP)=0. 
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
C
      IF(SUMPE.GT.1.E-9)THEN 
        RFACE=-SUMNE/SUMPE 
      ELSE 
        RFACE=1. 
      ENDIF 
C
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1) RFACE=1. 
C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
        DO L=1,LMP 
          DEP=DEL(L) 
          IF(DEP.GE.0.) DEP=DEP*RFACE 
          e3(l)=e3(l)+dep 
        ENDDO 
C
      q2(i,j,1)=max(e3(1)+e3(1)-epsq2,epsq2)*hbm 
     1         +q2(i,j,1)*(1.-hbm)
      DO L=2,LMP-1 
      q2(i,j,l)=max(e3(l)+e3(l)-q2(i,j,l-1),epsq2)*hbm
     1         +q2(i,j,l)*(1.-hbm)
      ENDDO 
C----------------------------------------------------------------------- 
  200 CONTINUE
C*********************************************************************** 
                             RETURN 
                             END 
