      subroutine eta2hybspline(ihinx,ihiny,etop,hf,to,qo,uo,vo,cwmo,q2o
     &, ui,vi,ti,qi,q2i,cldlin)
!-----------------------------------------------------------------------
C       SUBPROGRAM DOCUMENTAION BLOCK
C
C  SUBROUTINE: ETA2HYB  ETA-TO-HYBRID INTERPOLATION
C
C
C  ABSTRACT:  VERTICALLY INTERPOLATES ETA DATA TO LAYERS.
C             PROGRAM ASSUMES HORIZONTAL INTERPOLATION HAS BEEN
C             PERFORMED EARLIER.
C
C  REFERENCE:
!    02-03-11  H CHUANG MODIFIED TO INTERPOLATE GEOPOTENTIAL
!              AND WIND USING SPLINE, Q AND CWD USING LN(Q)/LN(P)
!    01-07-01  Z. Janjic modified for pressure-sigma hybrid
C    01-10-31  H CHUANG DEBUG AND MODIFIED FOR OPERATIONAL USE
C     IHIN     - INDEX OF THE H-POINT ON THE INPUT GRID CLOSEST TO 
C                THE H-POINT ON THE OUTPUT GRID
C     ETOP     - PRESSURE AT THE MODEL TOP IN MB
C
C    CALLED FROM PROGRAM INTRST
C
C---------------------------------------------------------------------
      INCLUDE "parmeta.resin"
      INCLUDE "parmeta.resout"
      INCLUDE "parmsoil"
      INCLUDE "mpif.h"
      INCLUDE "MPI_PARM.comm"
C---------------------------------------------------------------------
C       IMJM=Number of grid points on output eta grid
C       LMI=Number of vertical levels on input eta grid
C       LMO=Number of vertical levels on output eta grid
C
                             P A R A M E T E R
     1 (IM=IMO,JM=JMO
     3, LMIP=LMI+1,LMIM=LMI-1,LMOP=LMO+1,LMOM=LMO-1
     4, KB=JM+2*IM-3)
c     6, LP1=LMO+1,LONG=IMJM*(6*LMO+3)+KB*(4*LMO*2+2)+2*LMO+LMOP)
C
                             P A R A M E T E R
     1 (CM1=2937.4,CM2=4.9283,CM3=23.5518,EPS=0.622
     2, RD=287.04,G=9.806,RG=1./G,EPSQ=1.E-12,EPSQ2=2.E-2
     3, B1=11.87799326,FCTR=0.75)
C--------------------------------------------------------------------
      PARAMETER(NBC=55)      
                             L O G I C A L
     1 RUN,FIRST,HF,NEST,same
C
                             C H A R A C T E R
     1 LABEL*32
C--------------------------------------------------------------------
                             C O M M O N /DAT/
     1 IDAT(3) 
C-----------------------------------------------------------------------
C
C   OUTETA CONTAINS THOSE VARIABLES VERTICALLY INTERPOLATED TO THE
C   LMO ETA LEVELS OF THE OUTPUT GRID
C
                             C O M M O N /OUTETA/
     1 PDO(IMO,JMO),PDB(KB,2)
     5,TB (KB,LMO,2),QB (KB,LMO,2),UB (KB,LMO,2),VB (KB,LMO,2)
     6,PT,DETAO(LMO),AETAO(LMO),ETAO(LMOP),DFLO(LMOP)
     7,SM(IMO,JMO),RESO(IMO,JMO),FISO(IMO,JMO)
C--------------------------------------------------------------------
C   MSKIN IS THE TERRAIN MASKS FOR THE INPUT GRID
C   HTMI,VTMI : INPUT GRID HEIGHT MASK AT H,V-POINTS (0=Below ground,
C                 1=above ground)
C   LMHI,LMVI : INDEX OF THE LOWEST INPUT GRID ETA LEVEL ABOVE
C                 GROUND
C
                             C O M M O N /MSKIN/
     1 HTMI(IMI,JMI,LMI),VTMI(IMI,JMI,LMI),LMHI(IMI,JMI)
     2 ,LMVI(IMI,JMI)
C--------------------------------------------------------------------
c                             d i m e n s i o n
      common /HYBCOORD/ pdtop,dsg(lmo),sgml(lmo),sgm(lmo+1)
     &,dsg1(lmo),sgml1(lmo),sg1(lmo+1)
     &,dsg2(lmo),sgml2(lmo),sg2(lmo+1)
!----------------------------------------------------------------------
                             D I M E N S I O N
     1  ETAI(LMIP), LDT1(LMO),DFLI(LMIP)
     2, DETAI(LMI),AETAI(LMI),PMVP(LMI)
     3, PINTI(IM,JM,LMIP),PMIDI(IM,JM,LMI)
c     & ,ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP),DLT(LMO)
     4,ZETAI(IM,JSTA:JEND,LMIP),ZETAO(IM,JSTA:JEND,LMOP)
c     6, ZSFCO(IM,JM),REFO(IM,JM),dum3di(im,jm,lmi)
     7, IHINX(IM,JM),IHINY(IM,JM),PDSLI(IM,JM),zsfcoeta(imi,jmi)
     8, LMHOI(IM,JM),HTMOI(IM,JSTA:JEND,LMI)
     9, PSNEW(IM,JM),TLEV(LMIP),LEVATM(IM,JM),ZSFCB(KB)
     +, dum2do(IM,JM)
     1, TO(IM,JSTA:JEND,LMO),QO(IM,JSTA:JEND,LMO)
     2, UO(IM,JSTA:JEND,LMO),VO(IM,JSTA:JEND,LMO)
     3, CWMO(IM,JSTA:JEND,LMO),Q2O(IM,JSTA:JEND,LMO)
     4, ui(IM,JSTA:JEND,LMI),vi(IM,JSTA:JEND,LMI)
     5, ti(IM,JSTA:JEND,LMI),qi(IM,JSTA:JEND,LMI)
     6, q2i(IM,JSTA:JEND,LMI),cldlin(IM,JSTA:JEND,LMI)
C     +, Q2B(KB,LMO,2),CWMB(KB,LMO,2)
Cmp (try more precision?)
      real pcol(lmi+2),zcol(lmi+2),pmcol(lmi+1),qcol(lmi)
     +,ccol(lmi),ucol(lmi+1),vcol(lmi+1)
        REAL(8) ALPETA, ARGLOG, DLT(LMO), ALPSFC, ARGLG2
        REAL(8) ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP)
        REAL(8) ALPUB,ALPLB, CLOGES, ESE
C-----------------------------------------------------------------------
      INTEGER IUN1,IUN2,IUN3	
      DATA IUN1 /16/,IUN2 /17/,IUN3 /18/
C-----------------------------------------------------------------------
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C-----------------------------------------------------------------------
                         INCLUDE "TEMPRES"
C***
C***  DEFINITION OF CONSTANTS NEEDED FOR VERTICAL INTERPOLATION
C***

      CP=1004.6
      P00=1.E5
      GAMMA=0.0065
      PRF0=101325.
      T0=288.
      GORG=G/(RD*GAMMA)
      RGOG=1./GORG
      ROG=RD/G
      CPOG=CP/G
      CAPA=RD/CP
      PT=ETOP*100.
      ALPT=ALOG(PT)
C-------------------------------------------------------------
      REWIND IUN1
      REWIND IUN2
      REWIND IUN3
!
!  DETAI = ETA(L)-ETA(L-1) FOR THE INPUT GRID
!  ZSFCO = SIGMA HEIGHT ON OUTPUT GRID
!  SM = LAND/SEA MASK ON OUTPUT GRID
!  REFO = VALUE OF SIGMA AT THE SURFACE ON THE OUTPUT GRID=1
!
      READ(IUN1)DETAI      
C---------------------------------------------------------------------
C***
C***  FOR A ETA-TO-ETA
C***
c     READ(IUN3)FISO,SM
c     READ(IUN3)RESO
C---------------------------------------------------------------------
C***
C***  FOR A ETA-TO-HYBRID
C***
      READ(IUN3)FISO,DUM2DO,SM
C
C  examine eastern bdy of west nest
      if(mype.eq.0)then
      do j=1,jm
       iii=im+mod(j,2)-1
       if(abs(sm(iii,j)-1.0).lt.0.1)
     +print*,'water at eastern bdy of west nest at',iii,j
      end do
      end if
CC WRITING BOUNDARY TERRAIN FOR MAKING BOUNDARY TENDENCIES 
      CALL COLLECT(FISO)
      IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IMO
        ZSFCB(N)=FISO(I,1)
c       if(zsfcb(n).ne.0.)print*,'mountainous bdy'
        N=N+1
       END DO
       DO I=1,IMO
        ZSFCB(N)=FISO(I,JM)
c       if(zsfcb(n).ne.0.)print*,'mountainous bdy'
        N=N+1
       END DO
       DO J=3,JMO-2,2
        ZSFCB(N)=FISO(1,J)
c       if(zsfcb(n).ne.0.)print*,'mountainous bdy'
        N=N+1
       END DO
       DO J=3,JMO-2,2
        ZSFCB(N)=FISO(IM,J)
c       if(zsfcb(n).ne.0.)print*,'mountainous bdy'
        N=N+1
       END DO 
       WRITE(60)ZSFCB
      END IF 
c MAKE SURE REFO=1 LMHO=LMO HTMO=1 FOR SIGMA COORDINATE
        DO J=JSTA,JEND
	 DO I=1, IMO
	  RESO(I,J)=1.0
	 END DO
	END DO    	  
C
C     CONVERT REFO TO RESO FOR CNSTS AND ZSFCO TO FIS
C   
      DO J=JSTA,JEND
       DO I=1,IM
        FISO(I,J)=FISO(I,J)*G
       END DO	
      ENDDO
!-----------------------------------------------------------------------
!***
!***  compute etai at the interfaces of the model layers
!***
      etai(1)=0.
          do l=2,lmi+1
      etai(l)=etai(l-1)+detai(l-1)
          enddo
      etai(lmi+1)=1.
      detai(lmi)=1.-etai(lmi)
!***
!***  compute aetai in the middles of the layers
!***
          do l=1,lmi
      aetai(l)=0.5*(etai(l)+etai(l+1))
          enddo
!-----------------------------------------------------------------------
!23456789112345678921234567893123456789412345678951234567896123456789712
!  This is the first time the hybrid coordinate appears in the job
!  stream, and that is why it is defined here.  Away from the mountains,
!  the depths of the layers may be the same as those of the input sigma
!  coordinate, or may be defined independently.
!-----------------------------------------------------------------------
      same=.true.
      if(same) then
!-----------------------------------------------------------------------
       READ(IUN2)DSG
       DO L=1,LMO
        DETAO(L)=DSG(L)
       END DO
       SGM(1)=0.0
       DO L=2,LMO+1
        SGM(L)=SGM(L-1)+DSG(L-1)
        ETAO(L)=SGM(L)
       END DO
       SGM(LMO+1)=1.0
       ETAO(LMO+1)=1.0
       DSG(LMO)=1.0-SGM(LMO)
       DETAO(LMO)=1.0-ETAO(LMO)
       DO L=1,LMO
        SGML(L)=0.5*(SGM(L)+SGM(L+1))
        AETAO(L)=SGML(L)
       END DO 
!-----------------------------------------------------------------------
      else
!-------computing of sg levels according to empirical formula-----------
       x0=0.10*pi
       x1=(1.300)*pi
       dx=(x1-x0)/(lmo-1)
!
       aa=1.
       bb=-(aa-1.)/(x1-x0)
!
       x=x0-dx
       sum=0.
       do l=1,lmo
        x=x+dx
        dsgl=aa+bb*(x-x0)+sin(x)
        dsg(l)=dsgl
        sum=sum+dsgl
       enddo
       rsum=1./sum
       sgm(1)=0.
       do l=1,lmo-1
        dsg(l)=dsg(l)*rsum
        sgm(l+1)=sgm(l)+dsg(l)
       enddo
       sgm(lmo+1)=1.
       dsg(lmo)=sgm(lmo+1)-sgm(lmo)
       do l=1,lmo
        sgml(l)=(sgm(l)+sgm(l+1))*0.5
       enddo
!-----------------------------------------------------------------------
      endif
!-----------------------------------------------------------------------
!  ptsgm is the approximate pressure at the boundary between pressure 
!  and sigma in Pa.
!-----------------------------------------------------------------------
      ptsgm=42000  ! Pa
      lpt2=0
      do l=1,lmo+1
       pl=sgm(l)*(101300.-pt)+pt
       if(pl.lt.ptsgm) lpt2=l
      enddo
      if(lpt2.gt.0) then
       pt2=sgm(lpt2)*(101300.-pt)+pt
      else
       pt2=pt
      endif
      if(mype.eq.0) then
       print*,'*** Sigma system starts at ',pt2,' Pa, from level ',lpt2
      endif
      pdtop=pt2-pt
      do l=1,lmo
       if(l.le.lpt2) then
        dsg1(l)=dsg(l)
       else
        dsg2(l)=dsg(l)
       endif
      enddo
!-----------------------------------------------------------------------
      sg1(1)=0.
      if(lpt2.gt.0) then
       do l=1,lpt2
        sg1(l+1)=sg1(l)+dsg1(l)
       enddo
       do l=2,lpt2
        sg1(l)=sg1(l)/sg1(lpt2+1)
       enddo
       sg1(lpt2+1)=1.
       do l=lpt2+2,lmo+1
        sg1(l)=1.
       enddo
       do l=1,lpt2
        dsg1(l)=sg1(l+1)-sg1(l)
        sgml1(l)=(sg1(l)+sg1(l+1))*0.5
       enddo
      endif
      do l=lpt2+1,lmo
       dsg1(l)=0.
       sgml1(l)=1.
      enddo
!
      do l=1,lpt2+1
       sg2(l)=0.
      enddo
      do l=lpt2+1,lmo
       sg2(l+1)=sg2(l)+dsg2(l)
      enddo
      do l=lpt2+2,lmo
       sg2(l)=sg2(l)/sg2(lmo+1)
      enddo
      sg2(lmo+1)=1.
      do l=lpt2+1,lmo
       dsg2(l)=sg2(l+1)-sg2(l)
       sgml2(l)=(sg2(l)+sg2(l+1))*0.5
      enddo
      do l=1,lpt2
       dsg2(l)=0.
       sgml2(l)=0.
      enddo
!-----------------------------------------------------------------------
      if(mype.eq.0)then
       write(*,*)'dsg=', dsg
       write(*,*)'sgm=', sgm
       write(*,*) 'sgml= ',sgml
       write(*,*) 'dsgl= ',dsg1
       write(*,*) 'sgl= ',sg1
       write(*,*) 'sgml1= ',sgml1
       write(*,*) 'dsg2= ',dsg2
       write(*,*) 'sg2= ',sg2
       write(*,*) 'sgml2= ',sgml2
      end if
!-----------------------------------------------------------------------
C----------------------------------------------------------------
C***
C***  COMPUTE THE HEIGHTS OF THE STANDARD INTERFACES
C***  FOR THE INPUT DOMAIN
C***
      DO L=1,LMIP
        DFLI(L)=G*T0*(1.-((PT+ETAI(L)*(PRF0-PT))/PRF0)**RGOG)/GAMMA
      ENDDO
      if(mype.eq.0)then
       do l=1,lmip 
        print*,'STANDARD GEO HEIGHT AT L=',l,',=',DFLI(L)
       end do
      end if
      IF(HF)THEN
       DO J=JSTA,JEND
       DO I=1,IM
        LMHOI(I,J)=LMHI(I,J)
       END DO
       END DO
      ELSE
       DO 50 J=JSTA,JEND
        DO 50 I=1,IM
         LMHOI(I,J)=LMI
         DO L=2,LMI
          IF(TI(I,J,L).LT.1.)THEN
           LMHOI(I,J)=L-1
           GO TO 50
          ENDIF
         ENDDO
   50  CONTINUE
      END IF
CCCCC
      IF(HF)THEN
       DO J=JSTA,JEND
        DO I=1,IM
	 DO L=1,LMI
	  HTMOI(I,J,L)=HTMI(I,J,L)
	 END DO
	END DO
       END DO
      ELSE 	        
       DO 60 J=JSTA,JEND
        DO 60 I=1,IM
         DO L=2,LMIP
          HTMOI(I,J,L-1)=1.
          IF((FISO(I,J)-DFLI(L)).GT.1.E-2.AND.DFLI(L).GT.1.E-2)THEN
           DO LL=L,LMI
             HTMOI(I,J,LL)=0.
           ENDDO
           GO TO 60
          ENDIF
         ENDDO
   60  CONTINUE
      END IF  
CC testing 
      DO J=JSTA,JEND
       DO I=1,IM
c        IF(fisi(i,j).ge.fiso(i,j))then
         DO L=1,LMHOI(I,J)
          HTMOI(I,J,L)=1.
         END DO
         DO L=LMHOI(I,J)+1,LMI
          HTMOI(I,J,L)=0.
         END DO
c        else if (fisi(i,j).lt.fiso(i,j))then
c         do l=1,lmi
c          if(htmoi(i,j,l).lt.0.1)then
c           lmhoi(i,j)=l-1
c           go to 62
c          end if
c         end do
c 62      continue
c        end if
       END DO
      END DO
C     
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
c
c     if(jj.ge.jsta.and.jj.le.jend)then
c      do l=1,lmi
c       print*,'DEBUG ETA2HYBSPLINE:L,HTMOI= ',l,htmoi(ii,jj,l)
c      end do
c     end if
C
C----------------------------------------------------------------
C***
C***  COMPUTE HEIGHTS OF THE INPUT ETA INTERFACES
C***
cwas      allocate(ZETAI(IM,JM,LMIP))
      DO J=JSTA,JEND
       DO I=1,IM
        PDSLI(I,J)=RESI(I,J)*PDI(I,J)    
       END DO
      ENDDO
c      PRINT*,'LMHISM,AND ZSFC(207,7)= ',LMHI(207,7),SM(207,7)
c     + ,ZSFCO(207,7)
      
C       
C***
C***  IF THE OUTPUT POINT IS ABOVE GROUND BUT THE INPUT POINT
C***  IS BELOW GROUND THEN:
C***    (1) IF ADJACENT INPUT POINTS ARE ABOVE GROUND THEN USE
C***        A WEIGHTED AVERAGE OF THOSE ABOVE-GROUND INPUT
C***        TEMPERATURES
C***    (2) IF NO ADJACENT INPUT POINTS ARE ABOVE GROUND THEN
C***        EXTRAPOLATE DOWNWARD FROM THE TWO INPUT LAYERS
C***        ABOVE GROUND AND USE AN ARBITRARY FACTOR TO CHANGE THE
C***        EXTRAPOLATION FROM PURELY LINEAR (FCTR=1 IS LINEAR)
C***
      DO L=1,LMI
         AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
      ENDDO
CC EXTRAPOLATE INPUT TEMPERATURE DOWNWARD 1 TO 3 LAYERS IN CASE
CC SIGMA TERRAIN FALLS UNDER ETA TERRAIN
c      DO J=JSTA,JEND
c       DO I=1,IM
c        LMHIK=LMHOI(I,J)
c        IF(LMHIK .LT. LMI)THEN
cc        IF((FISO(I,J)/g).LE.ZSFCOETA(I,J))then
c         APEL1=ALOG(AETAI(LMHIK-5)*PDSLI(I,J)+PT)
c         APEL2=ALOG(AETAI(LMHIK)*PDSLI(I,J)+PT)
c         LEND=MIN(LMI,LMHIK+4)
c         DO L=LMHIK+1,LEND
c          APEL3=ALOG(AETAI(L)*PDSLI(I,J)+PT)
c          TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-
c     1           TI(I,J,LMHIK-5))*(APEL3-APEL2)/(APEL2-APEL1)
Cc USE STANDARD LAPSE RATE TO EXTRAPOLATE TEMPERATURE
cc         TI(I,J,L)=TI(I,J,L-1)+((AETAI(L)-AETAI(L-1))*PDSLI(I,J))
cc     1   *TI(I,J,L-1)/GORG/(PT+ETAI(L)*PDSLI(I,J))
c          HTMOI(I,J,L)=1.0
c         END DO
c        END IF
c       END DO
c      END DO
C
      DO L=LMI,1,-1
        DO J=JSTA,JEND
	 DO I=1,IM
	  ZETAI(I,J,LMIP)=0.0
	  APEL3=AETAI(L)*PDSLI(I,J)+PT
	  DPDE=DETAI(L)*PDSLI(I,J)
	  IF(HF)GO TO 61
	  IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
           HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1          +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
          ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
           HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     1          +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
	  ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
            HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J),IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          ELSE
	    HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          END IF
	  LMHIK=LMHOI(I,J)
	  IF(HWGT.LT.0.5 .AND.HTMOI(I,J,L).GT.0.5)THEN
            APEL1=AETAI(LMHIK-5)*PDSLI(I,J)+PT
            APEL2=AETAI(LMHIK)*PDSLI(I,J)+PT
            TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-
     1      TI(I,J,LMHIK-5))*(APEL3-APEL2)/(APEL2-APEL1)*FCTR
	  ENDIF	  
 61       CONTINUE	  
	  RTOP=RD*TI(I,J,L)*(1.+0.608*QI(I,J,L))/APEL3
          FI=G*ZETAI(I,J,L+1)+RTOP*DPDE
          IF(HWGT.GT.0.5)HWGT=1.
          ZETAI(I,J,L)=((FI-DFLI(L))*HTMOI(I,J,L)+DFLI(L))*RG	  
         END DO
	ENDDO
      ENDDO	
C***
C***  FIND THE PRESSURE ON THE INPUT INTERFACES
C***
      DO J=JSTA,JEND
       DO I=1,IM
        PINTI(I,J,1)=PT
       END DO
      ENDDO
C
      DO L=2,LMIP
        DO J=JSTA,JEND
	 DO I=1,IM
          PINTI(I,J,L)=PDSLI(I,J)*ETAI(L)+PT
         END DO
        ENDDO
      ENDDO
C***
C***  FIND THE PRESSURE AT THE INPUT MIDLAYER LOCATIONS
C***
      DO L=1,LMI
        DO J=JSTA,JEND
	 DO I=1,IM
          PMIDI(I,J,L)=PINTI(I,J,L)+0.5*DETAI(L)*PDSLI(I,J)
         END DO
	ENDDO
      ENDDO
c
C
C Using cubic spline to interpolate t, cwm
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
C
      do j=jsta,jend
c       do i=1,im+mod(j,2)-1
       do i=1,im
        if(i.eq.ii.and.j.eq.jj)then
         print*,'DEBUG ETA2HYBSPLINE: INPUT PS= ',pt+pdi(i,j),
     +      mype
         do l=1,lmi+1
          print*,'DEBUG ETA2HYBSPLINE :L,PINTI,ZETAI= '
     +	 ,L,PINTI(i,j,L),ZETAI(i,j,l),mype
	 end do
         do l=1,lmi
          print*,'DEBUG ETA2HYBSPLINE :L,PMID,TI,QI,cldlin= '
     +   ,L,PMIDI(i,j,L),TI(i,j,l),QI(i,j,l),cldlin(i,j,l),mype
         end do
	end if  
        lmact=lmhoi(i,j)
c        lmout=lmi
c        do l=1,lmi
c         if(abs(htmoi(i,j,l)-0.).lt.1.0e-10)then
c          lmout=l-1
c          go to 427 
c         end if
c        end do
c 427    continue
        do l=1,lmi+2
         pcol(l)=0.
         zcol(l)=0.
         if(l.le.(lmi+1))pmcol(l)=0.
        end do
c
        do l=1,lmip
         pcol(l)=pinti(i,j,l)
         zcol(l)=zetai(i,j,l)
        end do
        do l=1,lmi
         pmcol(l)=pmidi(i,j,l)
c         qcol(l)=qi(i,j,l)
c         ccol(l)=cldlin(i,j,l)
        end do
c        call hint(i,j,lmi,lmact,lmout,fisi(i,j)/g,pcol(1:lmip)
c     +  ,pmcol(1:lmi),zcol(1:lmip),qcol(1:lmi)
c     +  ,ccol(1:lmi),lmo,FISO(I,J)/G	
c     +  ,pdo(i,j),pdtop,pt,sgml1,sgml2,sg1,sg2,to(i,j,1:lmo)
c     +  ,qo(i,j,1:lmo),cwmo(i,j,1:lmo) )
c     
        call hint(i,j,lmi,lmact,fisi(i,j)/g,pcol
     +  ,pmcol,zcol
     +  ,lmo,FISO(I,J)/G	
     +  ,pdo(i,j),pdtop,pt,sgml1,sgml2,sg1,sg2,to(i,j,1:lmo))
        if(i.eq.ii.and.j.eq.jj)then
         do l=1,lmo
          print*,'DEBUG AFTER CALL hint: TVO= ',l,to(i,j,l),mype
         end do
        end if
c
        if(abs(sm(i,j)-1.).lt.0.01 .and. fisi(i,j)/g.lt.0.1
     +  .and. FISO(I,J)/G.lt.0.1)then
         if(abs(pdo(i,j)+pt+pdtop-pcol(lmi+1)).gt.50)print*,'pdi ne pdo'
     +,i,j,pcol(lmi+1),pdo(i,j)+pt+pdtop
        end if
       end do
      end do 
C      
C Interpolate log(q) with respect to log(p)
      DO 350 J=JSTA,JEND
       DO 350 I=1,IM
C
       DO LD=1,LMIM
        ALPR(LD)=ALOG(PMIDI(I,J,LD+1)/PMIDI(I,J,LD))
       ENDDO
       DO LD=1,LMI
        ALP(LD)=ALOG(PMIDI(I,J,LD))
       ENDDO
C
       LKO=LMO
       DO 340 L=1,LKO
       alpeta=log(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j))
       pmido=pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)
       DO 330 LD=2,LMI
        IF(HF)THEN 
	 HWGT=HTMI(I,J,LD)
	ELSE 
         IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
          HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
         ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
          HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD) 
	 ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
          HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         ELSE
	  HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         END IF
	END IF
        IF(HWGT.GT.0.5.AND.(ALPETA.LE.ALP(LD).OR.
     1                 (ALPETA.GT.ALP(LD).AND.LD.EQ.LMI)))THEN
        QLD=QI(I,J,LD)
C  put in fix for the situation when qi=0.
        QLD=AMAX1(QLD,EPSQ)  ! to avoid computing log(0.)
        CWMLD=CLDLIN(I,J,LD)
        CWMLD=AMAX1(CWMLD,EPSQ)  ! to avoid computing log(0.)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
C interpolate log(q) with respect to log(p)
        QI(I,J,LDM1)=AMAX1(QI(I,J,LDM1),EPSQ)  ! to avoid computing log(0.) 
        QETAIJ=EXP(ALOG(QLD)+(ALOG(QI(I,J,LDM1))-ALOG(QLD))*CF)
        if(QI(I,J,LDM1).le.0.)print*,'BAD QI,i,j,ld,qi,qo',i,j
     +, ld,QI(I,J,LDM1),QETAIJ
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        QSX=EPS*ESE/(PMIDO-ESE*(1.-EPS))
        QSMX=0.98*QSX
        QO(I,J,L)=AMIN1(QETAIJ,QSMX)
        QO(I,J,L)=AMAX1(QO(I,J,L),EPSQ)
        CWMO(I,J,L)=AMAX1(CWMO(I,J,L),EPSQ)  ! to avoid computing log(0.)
        CWMO(I,J,L)=EXP(ALOG(CWMLD)+(ALOG(cldlin(I,J,LDM1))
     +  -ALOG(CWMLD))*CF)
        if(CWMO(I,J,L).lt.0.)print*,'converting negative cwmo to 0'
     +  ,i,j,l,CWMO(I,J,L),mype
        CWMO(I,J,L)=AMAX1(CWMO(I,J,L),0.)
C       IF(QO(I,J,L).LT.QO(I,J,L-1).AND.LD.EQ.LMHO(I,J))QO(I,J,L)=QO(I,J,L-1)
        LKI=LD
        GO TO 340
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
C*** IF ALL NEIGHBORING POINTS (BRFORE HORIZONTAL INTERP)FOR THE 
C*** OUTPUT GRID ARE UNDERGROUND THEN THE INPUT QI IS ZERO 
C
      ELSEIF(HWGT.LT.0.5)THEN
c     if(TI(I,J,lki).lt.1.e-2)stop666
      if(TI(I,J,lki).lt.1.e-2)then
          qo(i,j,l)=qo(i,j,l-1)
          go to 340
      endif
        CLOGES=-CM1/TI(I,J,LKI)-CM2*ALOG10(TI(I,J,LKI))+CM3
        ESS=10.**(CLOGES+2.)
        QSLDM=EPS*ESS/(PMIDI(I,J,LKI)-ESS*(1.-EPS))
        RHLDM=QI(I,J,LKI)/QSLDM
        RHLDM=AMIN1(RHLDM,0.98)
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESL=10.**(CLOGES+2.)
        qsl=eps*esl/(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)-esl*(1.-eps))
        QO(I,J,L)=RHLDM*QSL
        CWMO(I,J,L)=0.
        GO TO 340
      ENDIF
  330 CONTINUE
  340 CONTINUE
C
      if(i.eq.ii.and.j.eq.jj)then
       do l=1,lmo
        print*,'DEBUG AFTER LOG(Q) INTERP :L,QO= '
     +	 ,L,QO(i,j,L),mype
       end do
      end if
      do l=1,lmo
       if(qo(i,j,l).gt.0.1.or.qo(i,j,l).lt.0.)
     +print*,'enormal q',i,j,l,qo(i,j,l),mype
      end do
      DO L=1,LMO
        IF(QO(I,J,L).LT.0.)QO(I,J,L)=0.
        IF(QO(I,J,L).GT.25.E-3)QO(I,J,L)=25.E-3
      ENDDO   
C
  350 CONTINUE
C
C***
C***  CONVERT FROM VIRTUAL TO TRUE TEMPERATURE.
C***
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
c
      DO 375 L=1,LMO
      DO 375 J=JSTA,JEND
      DO 375 I=1,IM
      IF(TO(I,J,L).GT.325. .and. i.le.(im+mod(j,2)-1))THEN
       PRINT*,'TEMP > 325,I,J,TO,FISI,FISO',I,J,L,TO(I,J,L)
     +,fisi(i,j)/g,fiso(i,j)/g,mype
       TO(I,J,L)=325.
      END IF 
      IF(TO(I,J,L).LT.150.)TO(I,J,L)=150.
      TV=TO(I,J,L)
      DO ITER=1,3
        TO(I,J,L)=TV/(1.+0.608*QO(I,J,L))
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        denom2=pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)-ese*(1.-eps)
        QSX=EPS*ESE/DENOM2
        QSMX=0.98*QSX
        if(i.eq.ii.and.j.eq.jj)then
          print*,'DEBUG : TO= ',l,to(i,j,l)
        end if
        IF(QO(I,J,L).LT.QSMX)GO TO 375
        QO(I,J,L)=QSMX
      ENDDO
  375 CONTINUE
c  
      do l=1,lmo
       do j=JSTA,JEND
        do i=1,im+mod(j,2)-1
c        do i=1,im
	 if(to(i,j,l).gt.325.0.or.to(i,j,l).lt.150.)print*,
     +   'enormal temperature',i,j,l,to(i,j,l),mype
         if(qo(i,j,l).gt.0.1)print*,'enormal q 2nd ',i,j,qo(i,j,l)
         if(cwmo(i,j,l).gt.0.01.or.cwmo(i,j,l).lt.0.0)
     +    print*,'enormal cwmo',i,j,l
     +  ,cwmo(i,j,l),mype
        end do
       end do
      end do 		 
C CHECK ON ENORMAL LAPSE RATE
      DO L=1,LMO-1
       DO J=JSTA,JEND
        DO I=1,IM+mod(j,2)-1
c	 IF(L.GT.30.AND.ABS(TO(I,J,L)-TO(I,J,L+1)).GT.5.0)PRINT*,
c     + 'LARGE LAPSE RATE AT ',I,J,L,'DT= ',TO(I,J,L)-TO(I,J,L+1)
        END DO
       END DO
      END DO
C  
      do l=1,lmo
       do j=JSTA,JEND
c        do I=1,Imo+mod(j,2)-1
        do I=1,Imo
	 if(cwmo(i,j,l).gt.0.01)print*,'enormal cwmo',i,j,l
     +  ,cwmo(i,j,l),mype
        end do
       end do
      end do 		 
C 
C USing cubic spline to interpolate u,v
      CALL EXCH3D(PMIDI)
      CALL EXCH(PDO)
      do j=jsta,jend
       do i=1,im
        do l=1,lmo
         uo(i,j,l)=0.
         vo(i,j,l)=0.
        end do
       end do
      end do 
      do j=jsta,jend
       do i=1,im-MOD(j,2)
        lmact=lmi
        do l=1,lmi
         if(abs(ui(i,j,l)).lt.1.0e-10)then
          lmact=l-1
          go to 428
         end if
        end do
 428    continue
        do l=1,lmi
         IF(J .EQ. 1 .AND. I .LT. IM)THEN   !SOUTHERN BC
          pmcol(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
          IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
C
         ELSEIF(J.EQ.JM .AND. I.LT.IM)THEN   !NORTHERN BC
          pmcol(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
          IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
         ELSEIF(I .EQ. 1 .AND. MOD(J,2) .EQ. 0) THEN   !WESTERN EVEN BC
          pmcol(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
          IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
         ELSEIF(I .EQ. IM .AND. MOD(J,2) .EQ. 0) THEN  !EASTERN BC
          pmcol(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
          IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
C
         ELSE IF (MOD(J,2) .LT. 1)THEN
C***
C***  PMVP AT ALL INTERIOR POINTS
C***
          pmcol(L)=0.25*(PMIDI(I,J,L)+PMIDI(I-1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
          IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I-1,J)+PDO(I,J+1)+
     1    PDO(I,J-1))
         ELSE
          pmcol(L)=0.25*(PMIDI(I,J,L)+PMIDI(I+1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
          IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I+1,J)+PDO(I,J+1)+
     1    PDO(I,J-1))
         END IF
c         pmcol(l)=pmidi(i,j,l)
         ucol(l)=ui(i,j,l)
         vcol(l)=vi(i,j,l)
        end do
        call vint(i,j,lmi,lmact,pmcol,ucol
     +,vcol
     +,lmo,pdvp,pdtop,pt,sgml1,sgml2
     +,uo(i,j,1:lmo),vo(i,j,1:lmo) )      
       end do
      end do 
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      do l=1,lmo  
       do j=jsta,jend
        do i=1,im
         if(abs(Uo(i,j,l)).gt.130.)print*,'enormal u',i,j,l
     +   ,Uo(i,j,l),mype
        end do 
       end do
      end do 
C***
C***  SOUTHERN V POINTS
C***
      DO L=1,LMO
       CALL COLLECT_LOC(UO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM-1
         UB(N,L,1)=DUM2DO(I,1)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN V POINTS
C
        DO I=1,IM-1
         UB(N,L,1)=DUM2DO(I,JM)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN V POINTS
C***
        DO J=2,JM-1,2
         UB(N,L,1)=DUM2DO(1,J)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN V POINTS
C***
        DO J=2,JM-1,2
         UB(N,L,1)=DUM2DO(IM,J)
         UB(N,L,2)=0.
         N=N+1
        END DO 
       END IF
      END DO 	
c
      do l=1,lmo  
       do J=JSTA,JEND
        do I=1,Im
         if(abs(Vo(i,j,l)).gt.130.)print*,'enormal v',i,j,l
     +   ,Vo(i,j,l),mype
        end do 
       end do
      end do
C***
C***  SOUTHERN V POINTS
      DO L=1,LMO
       CALL COLLECT_LOC(VO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM-1
         VB(N,L,1)=DUM2DO(I,1)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN V POINTS
C
        DO I=1,IM-1
         VB(N,L,1)=DUM2DO(I,JM)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN V POINTS
C***
        DO J=2,JM-1,2
         VB(N,L,1)=DUM2DO(1,J)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN V POINTS
C***
        DO J=2,JM-1,2
         VB(N,L,1)=DUM2DO(IM,J)
         VB(N,L,2)=0.
         N=N+1
        END DO 
       END IF
      END DO 		 
C***
C COLLECT PDB
      CALL COLLECT(PDO)
      IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IM
        PDB(N,1)=PDO(I,1)
        PDB(N,2)=0.
        N=N+1
       END DO
       DO I=1,IM
        PDB(N,1)=PDO(I,JM)
        PDB(N,2)=0
        N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(1,J)
        PDB(N,2)=0.
	N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(IM,J)
        PDB(N,2)=0.
	N=N+1
       END DO	
      END IF
C COLLECT TB
      DO L=1,LMO
       CALL COLLECT_LOC(TO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,1)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,JM)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(1,J)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(IM,J)
         TB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO 
C COLLECT QB
      DO L=1,LMO
       CALL COLLECT_LOC(QO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,1)
         QB(N,L,2)=0.  
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,JM)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(1,J)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(IM,J)
         QB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO
C      
C--------------------------------------------------------------------
C***
C***  INTERPOLATE Q2 LINEARLY IN LN(P) (BUT ON INTERFACES)
C***
      DO 450 J=JSTA,JEND
      DO 450 I=1,IM
C
      DO LD=1,LMI
        ALPR(LD)=ALOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
      ENDDO
      DO LD=1,LMIP
        ALP(LD)=ALOG(PINTI(I,J,LD))
      ENDDO
C
      LKO=LMO-1
      Q2SFC=AMAX1(B1**(2./3.)*USTAR(I,J)*USTAR(I,J),EPSQ2)
      Q2O(I,J,LKO+1)=Q2SFC
      alpsfc=alog(pdo(i,j)+pdtop+pt)
C
      DO 440 L=1,LKO
      alpeta=alog(pt+sg1(l+1)*pdtop+sg2(l+1)*pdo(i,j))
C
      DO 430 LD=2,LMI
       IF(HF)THEN
        HWGT=HTMI(I,J,LD)
       ELSE	
        IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
         HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
        ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
         HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
         HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1   IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2   +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE
	 HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1   IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2   +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        END IF
       END IF
       IF(HWGT.GT.0.5.AND.(ALPETA.LE.ALP(LD) .OR.
     1                     ALPETA.GT.ALP(LMI)))THEN
        Q2LD=Q2I(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        Q2IJ=Q2LD+(Q2I(I,J,LDM1)-Q2LD)*CF
        Q2TOP=AMAX1(Q2IJ,EPSQ2)
        ALPT=ALPETA
        LTOP=L
        Q2O(I,J,L)=Q2TOP
        GO TO 440
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME Q2 DECREASES LINEARLY TO Q2 AT THE NEW GROUND.
C***
       ELSEIF(HWGT.LT.0.5)THEN
        Q2DIF=Q2TOP-Q2SFC
        ALPDIF=ALPT-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LKO
          alpeta=log(pt+sg1(ll)*pdtop+sg2(ll)*pdo(i,j))
          Q2O(I,J,LL)=Q2TOP-Q2DIF*(ALPT-ALPETA)/ALPDIF
        ENDDO
        GO TO 450
      ENDIF
  430 CONTINUE
  440 CONTINUE
C
  450 CONTINUE
      DO L=1,LMO
       J=49
cwas   PRINT*,(Q2O(I,J,L),I=35,40)
      END DO   
CWAS      deallocate(dum3do)
C-----------------------------------------------------------
C***
C***  NOW REDEFINE PD TO HAVE IT EQUAL TO PSFC-PT.
C***
      DO J=JSTA,JEND
       DO I=1,IM
        PDO(I,J)=PDO(I,J)*RESO(I,J)
       END DO
      ENDDO
C
      DO L=1,LMOP
        dflo(l)=t0*(1.-((pt+sg1(l)*pdtop+sg2(l)*(prf0-pt2))
     &                  /prf0)**rgog)/gamma
      ENDDO
C----------------------------------------------------------------
C***
C***  TRANSLATE HBOT AND HTOP FROM INPUT LEVELS TO OUTPUT LEVELS
C***
      DO J=JSTA,JEND
       DO I=1,IM
        HBOTK=HBOT(I,J)
        HTOPK=HTOP(I,J)
        LMHOK=LMO
C
        DO 475 L=1,LMO
        IF(HBOTK.LT.LDT1(L))THEN
          HBOT(I,J)=L
C          IF(HBOT(I,J).GE.LMHOK-1)HBOT(I,J)=LMHOK-1
          GO TO 480
        ENDIF
  475   CONTINUE
C            
  480   DO 485 L=1,LMO
        IF(HTOPK.LT.LDT1(L))THEN
          HTOP(I,J)=L
          GO TO 485
        ENDIF
  485   CONTINUE
C            
       END DO
      ENDDO
!---------------------------------------------------------------------
CCC WRITING BOUNDARY CONDITIONS TO NBC FILE FOR BCTEND TO USE LATER
      IF(MYPE.EQ.0)THEN
       WRITE(NBC)RUN,IDAT,IHRST
       WRITE(NBC)PDB,TB,QB,UB,VB
      END IF 
!---------------------------------------------------------------------
      RETURN
      END
C
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c      subroutine hint
c     &(i,j,lmxi,lmi,lmout,hsi,pii,pmi,hi,qi,ci
c     &,lmo,hso,pdo,pdtop,pt,sgm1,sgm2,sg1,sg2
c     &,to,qo,co)
c
      subroutine hint
     &(i,j,lmxi,lmi,hsi,pii,pmi,hi
     &,lmo,hso,pdo,pdtop,pt,sgm1,sgm2,sg1,sg2
     &,to)
!-----------------------------------------------------------------------
! interpolation from eta representation into hybrid coordinate at mass
! points
!
! responsible persons: z. janjic, h. chuang
!
! lmxi    maximun number of eta layers in mass point columns
! lmi     number of active eta layers in mass point columns
! hsi     height of eta topography in m
! pii     pressure at interfaces of eta layers
! pmi     pressure in the middle of eta layers
! hi      height of the interfaces of eta layers in m
! qi      specific humidity at eta mid-layers
! ci      condensate at eta mid-layers
! lmo     number of hybrid layers
! hso     height of hybrid topography
! pdo     depth of hybrid sigma part
! pdtop   depth of the pressure part of the hybrid coordinate
! pt      pressure at the top of the model atmosphere
! sgm1    hybrid at mid-layers within the pressure part
! sgm2    hybrid at mid-layers within the sigma part
! sg1     hybrid at layer interfaces within the pressure part
! sg2     hybrid at layer interfaces within the sigma part
! to      temperature at hybrid mid-layers
! qo      specific humidity at hybrid mid-layers
! co      condensate at hybrid mid-layers
!-----------------------------------------------------------------------
      parameter
     &(rr=1./287.04)
!-----------------------------------------------------------------------
      dimension
     & pii(lmxi+2),hi(lmxi+2)
     &,pmi(lmxi+1),qi(lmxi+1),ci(lmxi+1)
     &,sgm1(lmo),sgm2(lmo),sg1(lmo+1),sg2(lmo+1)
     &,to(lmo),qo(lmo),co(lmo)
      real dum1d(lmi+1),dum2d(lmi+1)
      dimension
     & y2(lmxi+2),phld(lmxi+2),qhld(lmxi+2)
!---------local variables-----------------------------------------------
      dimension
     & ho(lmo+1),pmo(lmo),pio(lmo+1)
!
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
c
      if(i.eq.ii.and.j.eq.jj)then
       print*,'DEBUG in HINT:hsi,hso,pt,pdtop= '
     +,hsi,hso,pt,pdtop,mype
       do l=1,lmxi+1
        print*,'DEBUG in HINT:L,PINTI,ZETAI,sg1,sg2= '
     + ,L,PII(L),HI(L),sg1(l),sg2(l),mype
       end do
       do l=1,lmxi
        print*,'DEBUG in HINT:L,PMIDI,sgm1,sgm2= '
     +,L,pmi(l),sgm1(l),sgm2(l),mype
       end do
      end if     
!---------find surface pressure first-----------------------------------
c      psi=pii(lmi+1)
c          if(hsi.gt.hso) then ! extrapolate pressure from hsi to hso
c      d1=(pii(lmi+1)-pii(lmi))/(hsi-hi(lmi))
c      pso=(hso-hsi)*d1+psi
       psi=pii(lmi+1)
CC put in the following line to ensure pdi=pdo when fisi=fiso 
       if(abs(hi(lmi+1)-hso).lt.0.1)then
        hi(lmi+1)=hso
        pso=pii(lmi+1)
       else if(hi(lmi+1).gt.hso) then ! extrap pressure from hsi to hso
C compute surface pressure assuming p/z dependence
c      d1=(pii(lmi+1)-pii(lmi))/(hi(lmi+1)-hi(lmi))
c      pso=(hso-hi(lmi+1))*d1+psi
c      d1=(pii(lmi+1)-pii(lmi-4))/(hi(lmi+1)-hi(lmi-4))
c      pso=(hso-hi(lmi+1))*d1+psi
c compute surface pressure assuming log(p)/z dependence
      pso=exp(log(pii(lmi+1))+log(pii(lmi+1)/pii(lmi))*
     &(hso-hi(lmi+1))/(hi(lmi+1)-hi(lmi)))
      if(i.eq.ii.and.j.eq.jj)print*,'DEBUG in HINT:EXTRO PSO,lmhoi='
     +,lmi
          else ! interpolate 
          do l=1,lmxi+2
      y2(l)=0.
          enddo
      do l=1,lmi+1
       dum1d(l)=hi(lmi+2-l)
       dum2d(l)=pii(lmi+2-l)
      end do
cjjt  call spline2(i,j,lmi+1,dum1d(1:lmi+1),dum2d(1:lmi+1)
cjjt +,y2(1:lmi+1),1,hso,pso,phld(1:lmi+1),qhld(1:lmi+1))
      call spline2(i,j,lmi+1,dum1d,dum2d
     +,y2,1,hso,pso,phld,qhld)
      if(i.eq.ii.and.j.eq.jj)print*,'DEBUG in HINT:INTER PSO,lmhoi='
     +,lmi
          endif
!
      pdo=pso-pdtop-pt
!---------now output pressure levels can be defined---------------------
          do l=1,lmo
      pmo(l)=sgm2(l)*pdo+sgm1(l)*pdtop+pt
          enddo
          do l=1,lmo+1
      pio(l)=sg2(l)*pdo+sg1(l)*pdtop+pt
          enddo
!---------vertical reinterpolation of height at the output interfaces---
          do l=1,lmxi+2
      y2(l)=0.
          enddo
c          if(hsi.gt.hso) then ! 
          if(hi(lmi+1).gt.hso) then
      pii(lmi+2)=pso
      hi(lmi+2)=hso
!
cjjt  call spline2(i,j,lmi+2,pii(1:lmi+2),hi(1:lmi+2),y2(1:lmi+2)
cjjt +,lmo+1,pio(1:lmo+1),ho(1:lmo+1),phld(1:lmi+2),qhld(lmi+2))
      call spline2(i,j,lmi+2,pii,hi,y2
     +,lmo+1,pio,ho,phld,qhld)
          else
cjjt  call spline2(i,j,lmi+1,pii(1:lmi+1),hi(1:lmi+1),y2(1:lmi+1)
cjjt +,lmo+1,pio(1:lmo+1),ho(1:lmo+1),phld(1:lmi+1),qhld(1:lmi+1))
      call spline2(i,j,lmi+1,pii,hi,y2
     +,lmo+1,pio,ho,phld,qhld)
          endif
!---------vertical interpolation of moist variables at mid-layers-------
c          if(pmi(lmi).lt.pmo(lmo)) then
c      pmi(lmi+1)=pmo(lmo)
!
CC comment out spline interpolation of q for now
CC log(q)/log(p) interpolation is performed instead
c      d1=(qi(lmi)-qi(lmi-1))/(pmi(lmi)-pmi(lmi-1))
c      qi(lmi+1)=(pmo(lmo)-pmi(lmi))*d1+qi(lmi)
c          do l=1,lmxi+2
c      y2(l)=0.
c          enddo
c      call spline2(i,j,lmi+1,pmi(1:lmi+1),qi(1:lmi+1),y2(1:lmi+1)
c     +,lmo,pmo(1:lmo),qo(1:lmo),phld(1:lmi+1),qhld(1:lmi+1))
!
c      d1=(ci(lmi)-ci(lmi-1))/(pmi(lmi)-pmi(lmi-1))
c      ci(lmi+1)=(pmo(lmo)-pmi(lmi))*d1+ci(lmi)
c          do l=1,lmxi+2
c      y2(l)=0.
c          enddo
c      call spline2(i,j,lmi+1,pmi(1:lmi+1),ci(1:lmi+1),y2(1:lmi+1)
c     +,lmo,pmo(1:lmo),co(1:lmo),phld(1:lmi+1),qhld(1:lmi+1))
c          else
CC comment out spline interpolation of q for now
c          do l=1,lmxi+2
c      y2(l)=0.
c          enddo
c      call spline2(i,j,lmi,pmi(1:lmi),qi(1:lmi),y2(1:lmi)
c     +,lmo,pmo(1:lmo),qo(1:lmo),phld(1:lmi),qhld(1:lmi))
c          do l=1,lmxi+2
c      y2(l)=0.
c          enddo
c      call spline2(i,j,lmi,pmi(1:lmi),ci(1:lmi),y2(1:lmi)
c     +,lmo,pmo(1:lmo),co(1:lmo),phld(1:lmi),qhld(1:lmi))
c          endif
!---------now compute temperature---------------------------------------
          do l=1,lmo
      tv=(ho(l)-ho(l+1))*pmo(l)/(pio(l+1)-pio(l))
     +*rr*9.8
cc q interpolation is no longer done here so tv to t conversion will 
cc be done later    
c      to(l)=tv/(qo(l)*0.608-co(l)+1.)
      to(l)=tv
          enddo
!-----------------------------------------------------------------------
      if(i.eq.ii.and.j.eq.jj)then
       print*,'DEBUG in HINT:PSO =',pso
       do l=1,lmo+1
        print*,'DEBUG in HINT:L,PIO,HO= '
     + ,L,PIO(L),HO(L)
       end do
       do l=1,lmo
        print*,'DEBUG in HINT:L,PMO,TVO= '
     +,L,pmo(l),to(l)
       end do
      end if
C
      return
      end
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine vint
     &(i,j,lmxi,lmi,pmi,ui,vi
     &,lmo,pdo,pdtop,pt,sgm1,sgm2
     &,uo,vo)
!-----------------------------------------------------------------------
! interpolation from eta representation into hybrid coordinate at 
! velocity points
!
! responsible persons: z. janjic, h. chuang
!
! lmxi    maximun number of eta layers in mass point columns
! lmi     number of active eta layers in mass point columns
! pmi     pressure in the middle of eta layers
! ui      rotated u at eta mid-layers
! vi      rotated v at eta mid-layers
! lmo     number of hybrid layers
! pdo     depth of hybrid sigma part
! pdtop   depth of the pressure part of the hybrid coordinate
! pt      pressure at the top of the model atmosphere
! sgm1    hybrid at mid-layers within the pressure part
! sgm2    hybrid at mid-layers within the sigma part
! uo      rotated u at hybrid mid-layers
! vo      rotated v at hybrid mid-layers
!-----------------------------------------------------------------------
      dimension
     & pmi(lmxi+1),ui(lmxi+1),vi(lmxi+1)
     &,sgm1(lmo),sgm2(lmo)
     &,uo(lmo),vo(lmo)
      dimension
     & y2(lmxi+2),phld(lmxi+2),qhld(lmxi+2)
!---------local variables-----------------------------------------------
      dimension
     & pmo(lmo)
!---------output pressure levels----------------------------------------
          do l=1,lmo
      pmo(l)=sgm2(l)*pdo+sgm1(l)*pdtop+pt
          enddo
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
C H CHUANG, FIX FOR WHEN PMI(1).GT.PMO(1)
      if(pmi(1).gt.pmo(1))then
       d1=(pmo(1)-pmi(1))/(pmi(1)-pmi(2))
       ui(1)=ui(1)+d1*(ui(1)-ui(2))
       vi(1)=vi(1)+d1*(vi(1)-vi(2))
       pmi(1)=pmo(1)
      end if
      if(i.eq.ii.and.j.eq.jj)then
       print*,'DEBUG in VINT:pt,pdtop lmvoi= '
     +,pt,pdtop,lmi
       do l=1,lmxi
        print*,'DEBUG in VINT:L,PMI,UI,VI= '
     + ,L,PMI(L),UI(L),VI(L)
       end do
       do l=1,lmo
        print*,'DEBUG in VINT:L,PMO= '
     +,L,pmo(l)
       end do
      end if
!---------vertical interpolation of u and v at mid-layers---------------
          if(pmi(lmi).lt.pmo(lmo)) then
      pmi(lmi+1)=pmo(lmo)
!
c      d1=(ui(lmi)-ui(lmi-1))/(pmi(lmi)-pmi(lmi-1))
      d1=0.   
      if(i.eq.ii.and.j.eq.jj)print*,'LMI LMXI in VINT=',lmi,lmxi
      ui(lmi+1)=(pmo(lmo)-pmi(lmi))*d1+ui(lmi)
          do l=1,lmxi+2
      y2(l)=0.
          enddo
cjjt  call spline2(i,j,lmi+1,pmi(1:lmi+1),ui(1:lmi+1),y2(1:lmi+1)
cjjt +,lmo,pmo(1:lmo),uo(1:lmo),phld(1:lmi+1),qhld(1:lmi+1))
      call spline2(i,j,lmi+1,pmi,ui,y2
     +,lmo,pmo,uo,phld,qhld)
!
      if(i.eq.ii.and.j.eq.jj)then
       do l=1,lmo
        print*,'DEBUG1 in VINT:L,uo=',l,uo(l)
       end do
      end if 
c      d1=(vi(lmi)-vi(lmi-1))/(pmi(lmi)-pmi(lmi-1))
      d1=0.   !assume undergrounf temp to be vi(lmi)
      vi(lmi+1)=(pmo(lmo)-pmi(lmi))*d1+vi(lmi)
          do l=1,lmxi+2
      y2(l)=0.
          enddo
cjjt  call spline2(i,j,lmi+1,pmi(1:lmi+1),vi(1:lmi+1),y2(1:lmi+1)
cjjt +,lmo,pmo(1:lmo),vo(1:lmo),phld(1:lmi+1),qhld(1:lmi+1))
      call spline2(i,j,lmi+1,pmi,vi,y2
     +,lmo,pmo,vo,phld,qhld)
          else
          do l=1,lmxi+2
      y2(l)=0.
          enddo
cjjt  call spline2(i,j,lmi,pmi(1:lmi),ui(1:lmi),y2(1:lmi)
cjjt +,lmo,pmo(1:lmo),uo(1:lmo),phld(1:lmi),qhld(1:lmi))
      call spline2(i,j,lmi,pmi,ui,y2
     +,lmo,pmo,uo,phld,qhld)
      if(i.eq.ii.and.j.eq.jj)then
       do l=1,lmo
        print*,'DEBUG2 in VINT:L,uo=',l,uo(l)
       end do
      end if
          do l=1,lmxi+2
      y2(l)=0.
          enddo
cjjt  call spline2(i,j,lmi,pmi(1:lmi),vi(1:lmi),y2(1:lmi)
cjjt +,lmo,pmo(1:lmo),vo(1:lmo),phld(1:lmi),qhld(1:lmi))
      call spline2(i,j,lmi,pmi,vi,y2
     +,lmo,pmo,vo,phld,qhld)
          endif
!-----------------------------------------------------------------------
      return
      end
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine spline2(i,j,nold,xold,yold,y2,nnew,xnew,ynew,p,q)
!
!     ******************************************************************
!     *                                                                *
!     *  this is a one-dimensional cubic spline fitting routine        *
!     *  programed for a small scalar machine.                         *
!     *                                                                *
!     *  programer: z. janjic                                          *
!     *                                                                *
!     *  nold - number of given values of the function.  Must be ge 3. *
!     *  xold - locations of the points at which the values of the     *
!     *         function are given.  Must be in ascending order.       *
!     *  yold - the given values of the function at the points xold.   *
!     *  y2   - the second derivatives at the points xold.  If natural *
!     *         spline is fitted y2(1)=0. And y2(nold)=0. Must be      *
!     *         specified.                                             *
!     *  nnew - number of values of the function to be calculated.     *
!     *  xnew - locations of the points at which the values of the     *
!     *         function are calculated.  Xnew(k) must be ge xold(1)   *
!     *         and le xold(nold).                                     *
!     *  ynew - the values of the function to be calculated.           *
!     *  p, q - auxiliary vectors of the length nold-2.                *
!     *                                                                *
!     ******************************************************************
!
                             d i m e n s i o n
     2 xold(nold),yold(nold),y2(nold),p(nold),q(nold)
     3,xnew(nnew),ynew(nnew)
!-----------------------------------------------------------------------
c  east08 check point
c      ii=189
c      jj=441
c Mt. Mckinley
c      ii=108
c      jj=252
c  west08 check point
c      ii=195
c      jj=371
c      ii=98
c      jj=177
c  central08 check point
      ii=20
      jj=485
c
      if(nnew.eq.1)then
       if(i.eq.ii.and.j.eq.jj)then
        print*,'DEBUG in SPLINE2:HSO= ',xnew
        do l=1,nold
         print*,'DEBUG in SPLINE2:L,PINTI,ZETAI= '
     +  ,L,yold(L),xold(L)
        end do
       end if
      end if
      noldm1=nold-1
!
      dxl=xold(2)-xold(1)
      dxr=xold(3)-xold(2)
      dydxl=(yold(2)-yold(1))/dxl
      dydxr=(yold(3)-yold(2))/dxr
      rtdxc=.5/(dxl+dxr)
!
      p(1)= rtdxc*(6.*(dydxr-dydxl)-dxl*y2(1))
      q(1)=-rtdxc*dxr
!
      if(nold.eq.3) go to 700
!-----------------------------------------------------------------------
      k=3
!
 100  dxl=dxr
      dydxl=dydxr
      dxr=xold(k+1)-xold(k)
      dydxr=(yold(k+1)-yold(k))/dxr
      dxc=dxl+dxr
      den=1./(dxl*q(k-2)+dxc+dxc)
!
      p(k-1)= den*(6.*(dydxr-dydxl)-dxl*p(k-2))
      q(k-1)=-den*dxr
!
      k=k+1
      if(k.lt.nold) go to 100
!-----------------------------------------------------------------------
 700  k=noldm1
!
 200  y2(k)=p(k-1)+q(k-1)*y2(k+1)
!
      k=k-1
      if(k.gt.1) go to 200
!-----------------------------------------------------------------------
      k1=1
!
 300  xk=xnew(k1)
!
      do 400 k2=2,nold
      if(xold(k2).le.xk) go to 400
      kold=k2-1
      go to 450
 400  continue
      ynew(k1)=yold(nold)
      go to 600
!
 450  if(k1.eq.1)   go to 500
      if(k.eq.kold) go to 550
!
 500  k=kold
!
      y2k=y2(k)
      y2kp1=y2(k+1)
      dx=xold(k+1)-xold(k)
      rdx=1./dx
!
      ak=.1666667*rdx*(y2kp1-y2k)
      bk=.5*y2k
      ck=rdx*(yold(k+1)-yold(k))-.1666667*dx*(y2kp1+y2k+y2k)
!
 550  x=xk-xold(k)
      xsq=x*x
!
      ynew(k1)=ak*xsq*x+bk*xsq+ck*x+yold(k)
!
 600  k1=k1+1
      if(k1.le.nnew) go to 300
!-----------------------------------------------------------------------
      return
      end



      
